/*! file
	File:			HostPalm.c

	Description:	This is the host portion of the example.  OS and device specific code to get the input, 
					load graphics, draw specific graphics, save and load games, and so on belongs 
					in here for use by the game.
	
					This file also provides a TapWave look and feel to the game. This code runs like
					a TapWave app, and can override the game's wishes to provide a best handheld
					interpretation of the game.

					This file is in C like most Palm programs.  It interacts with game.cpp to show
					how to work with C++ programs.

	Author:			Roger Flores

	Copyright:		Copyright  2003 by Tapwave, Inc.

	Disclaimer:		IMPORTANT:  This Tapwave software is provided by Tapwave, Inc. ("Tapwave").  Your 
					use is subject to and governed by terms and conditions of the Software Development 
					Kit Agreement ("SDK Agreement") between you and Tapwave.  If you have not entered 
					into Tapwaves standard SDK Agreement with Tapwave, you have no right or license 
					to use, reproduce, modify, distribute or otherwise exploit this Tapwave software.  
					You may obtain a copy of Tapwaves standard SDK Agreement by calling 650-960-1817
					or visiting Tapwave at http://www.tapwave.com/developers/. 

	Change History (most recent first):
				
*/

/*!

Glossary

game cycle:  Everything needed to advance the game one step.  This includes
    taking input, moving all world objects, triggering events, collision 
    detection, and drawing.

offscreen buffer: A set of pixels not used by the screen.  The graphics 
    are composed there and when done, made visible by the screen, by 
    copying them, or sometimes by pointing the screen hardware to them.
    
landscape mode: The screen placed into 480x320 mode.  No status bar is
    visible.  The Palm coordinate system remains 240x160 (low density).
    
game menu: A UI system to control or configure the game.  The method used
    draws over game in the offscreen buffer.  An alternative is to use 
    the Palm Menu system, but if so, be careful not to draw the game 
    which would overwrite the menus.

Palm bitmaps: Bitmaps in a native Palm format.  Typically created from
    BMP files with pilrc or pasted into Constructor and stored in the
    app's PRC.  Bitmaps can have alternate color depths and resolutions and are 
    collectively called a bitmap family.  The most appropriate family
    member is drawn.  The Palm API usually uses the hardware acceleration 
    when possible.
*/

#include <TapWave.h>
#ifdef __PALMOS_ARMLET__
  #include "BannedTraps.h"
#endif
#include "host.h"
#include "utils.h"
#include "gauge.h"
#include <math.h>
#include "random.h"
#include "T3DLIB1_AK.h"
//#include "t3dlib1.h"
#include "FastGameRsc.h"
#include <TwMidi.h>
#include "nogfxsign.h"


// Define this to run full speed.  Leave undefined to run at a constant 
// frame rate.
//#define OPTION_DELAY_NONE 1


/*
 * UI and graphics for Palm OS software are often edited using the 
 * Constructor tool.  Constructor can be told to describe the UI and
 * graphic resources in a header file.  That header is included here.
 */


#define ObjectBitmapFirst       2000
#define ObjectBitmapCount		2



// List of key bindings
#define rotateLeftKey		keyBitHard1 // polled every game period
#define rotateRightKey		keyBitHard2 // polled every game period
#define shootKey			keyBitHard3 // polled every game period
#define thrustKey			keyBitHard4 // polled every game period
#define keysAllowedMask		(rotateLeftKey | rotateRightKey | shootKey | thrustKey | vchrTriggerRight | vchrTriggerLeft)


/* This is the input that we regularly poll for each game cycle.  More
could be added. */
typedef struct 
{
  Int32 joyX;
  Int32 joyY;
  Int32 ActionA;
  Int32 ActionB;
  Int32 ActionC;
  Int32 ActionD;
  Int32 TriggerA;
  Int32 TriggerB;
} AppJoystickInput;


// This defines the game menu, which is shown while the game is paused.
// It is reached by pressing the Home button.  Pressing Home again 
// switches apps to Home.  Pushing the joystick down selects the game
// resume.
#define GameMenuResume  0
#define GameMenuExit    1
#define GameMenuCount   2

// The ARM compiler does not link strings in a Position Independent code model.
// A workaround is to fully allocate the storage.
char GameMenuText[GameMenuCount][8] = 
{
    "Resume",
    "Exit"
};

#define FPS_DATA_MAX 300
typedef struct
{
	int time[FPS_DATA_MAX];
	int count;
	int time_of_last_data_out;
} FPSType;

typedef struct
{
    TwGfxSurfaceType * pictureBitmapP[ObjectBitmapCount];
    
    //! These are only used if customBlitting is true.  They cache some of the info on
    //! the pictures that will be used often.  Custom blitting means using 
    //! a custom blitter drawing to a frame buffer, versus using the standard
    //! API calls.
    TwGfxType * GFX;
    TwGfxSurfaceType * displaySurface;
    
    MemPtr pictureBitmapBitsP[ObjectBitmapCount];
    Boolean pictureIsTransparent[ObjectBitmapCount];
    Int16 _pad_;
    UInt32 pictureTransparencyIndex[ObjectBitmapCount];
    Boolean customBlitting;
    Boolean isInputMasked;

    UInt16 soundAmp;            // default sound amplitude

    Int32 screenWidth;
    Int32 screenHeight;
	
	// This is the offscreen buffer which is copied to the screen.
    TwGfxSurfaceType * screenBufferH;    // screenBuffer
    UInt8 * screenBufferP;  // DOLATER - This is really a pointer to the pixels!
    
    
	TwDeviceHandle rumblerH;	// device to play the rumble data
    
    //! These are used to get the input.  isInputMasked prevents them
    //! from being set or unset repeatedly.  The padding inserted is
	//! to achieve ARM alignment for these data structures.
	//! NOTE: replace with compiler options when CW9 is avaiable.
    AppJoystickInput joystickInput;
    TwInputHandle inputQueueP;
    Int32 *inputFormatP;
    
    //! Indicates which game menu item is selected.
    Int16 gameMenuSelected;
    Int16 pad5[1];
    
    //! Controls when the next cycle starts
    UInt32 cycleNextTime;
    UInt32 cycleDurationTime;
    
    //! Gauges
    GaugeFIFOType * SPFGauge;	// Seconds per frame
    GaugeFIFOType * FPSGauge;	// Frames per second
    FPSType fps;

} App;

static App GameApp;

	BOB          ship;             // the ship
	BOB          black_hole;       // the gravity well
	BOB			 Galaxy;				// background galaxy
	float particle_wind;    // assume it operates in the X direction
	float particle_gravity; // assume it operates in the Y direction
	// storage for our lookup tables
	float cos_look[361]; // 1 extra element so we can store 0-360 inclusive
	float sin_look[361];
	PARTICLE particles[MAX_PARTICLES]; // the particles for the particle engine
	UInt8 colour_range[24] = {125,143,161,226,179,197,     //red, blue, green, white, 6 each
						95,101,107,107,203,209,
						210,211,212,228,213,214,
						0,223,50,219,190,215};

	TwGfxPackedRGBType RGBColour_range[24] = { TwGfxComponentsToPackedRGB(255,0,0),TwGfxComponentsToPackedRGB(204,0,0),
		TwGfxComponentsToPackedRGB(153,0,0),TwGfxComponentsToPackedRGB(128,0,0),TwGfxComponentsToPackedRGB(102,0,0),TwGfxComponentsToPackedRGB(51,0,0),
		TwGfxComponentsToPackedRGB(0,0,255),TwGfxComponentsToPackedRGB(0,0,204),TwGfxComponentsToPackedRGB(0,0,153),TwGfxComponentsToPackedRGB(0,0,153),
		TwGfxComponentsToPackedRGB(0,0,102),TwGfxComponentsToPackedRGB(0,0,151),
		TwGfxComponentsToPackedRGB(0,255,0),TwGfxComponentsToPackedRGB(0,204,0),TwGfxComponentsToPackedRGB(0,153,0),TwGfxComponentsToPackedRGB(0,128,0),
		TwGfxComponentsToPackedRGB(0,102,0),TwGfxComponentsToPackedRGB(0,51,0),
		TwGfxComponentsToPackedRGB(255,255,255),TwGfxComponentsToPackedRGB(221,221,221),TwGfxComponentsToPackedRGB(153,153,153),TwGfxComponentsToPackedRGB(119,119,119),
		TwGfxComponentsToPackedRGB(51,51,51),TwGfxComponentsToPackedRGB(17,17,17)

	};
	TwGfxPackedRGBType BackgroundColour = TwGfxComponentsToPackedRGB(0,0,0);
	HOSTPARTICLE hostparticles[MAX_PARTICLES];  // The draw system for particles
	HOSTPARTICLE_COLOUR hostparticles_colour[MAX_PARTICLES];

BOB wraith;                   // the player 
BOB plasma[MAX_PLASMA];       // plasma pulses
BOB rocks[MAX_ROCKS];         // the asteroids
BOB rock_s, rock_m, rock_l;   // the master templates for surface info
BOB bursts[MAX_BURSTS];       // the explosion bursts
BOB gunships[MAX_GUNSHIPS];   // the gunships
BOB powerups[MAX_POWERUPS];   // the powerups
BOB stations[MAX_STATIONS];   // the starbase stations
BOB mines[MAX_MINES];         // the predator mines
BOB hud;                      // the art for the scanner hud
BOB stationsmall;             // small station bob

int rock_sizes[3] = {96,56,32}; // X,Y dims for scaler

STAR stars[MAX_STARS];          // the star field


// player state variables
int player_state       = PLAYER_STATE_ALIVE;
int player_score       = 0;   // the score
int player_ships       = 3;   // ships left
int player_regen_counter = 0; // used to track when to regen
int player_damage      = 0;   // damage of player
int player_ammo        = 1000;// ammo of player
int player_counter     = 0;   // used for state transition tracking
int player_regen_count = 0;   // used to regenerate player
int player_shield_count = 0;  // used to display shields
int ready_counter = 0,        // used to draw a little "get ready"
    ready_state   = 0;
int ammo_regen_counter = 0;
float mine_tracking_rate = 2; // rate that mines track player

int win_counter = 0,          // tracks if player won
    player_won = 0;   

int station_id = -1,          // uplink station id
    num_stations_destroyed = 0;

int intro_done = 0;           // flags if intro has played already

int game_paused      = 0,
    pause_debounce   = 0,
    huds_on          = 1,
    scanner_on       = 1,
    huds_debounce    = 0,
    scanner_debounce = 0;

// color palettes, so we don't have to reload all the time
//PALETTEENTRY         game_palette[256]; // holds the main game palette
//PALETTEENTRY         menu_palette[256]; // gee what do you think
//PALETTEENTRY         andre_palette[256]; // for me

// positional and state info for player
float player_x=0,
      player_y=0,
	  player_dx=0, 
      player_dy=0, 
	  player_xv=0, 
	  player_yv=0,
      vel = 0;

// these contain the virual cos, sin lookup tables for the 16 sector circle
float cos_look16[16],
      sin_look16[16];
      
int MusicPlaying = 0;
UInt32 MusicDuration;
UInt16 soundAmp;            // default sound amplitude

// sound id's
/*int intro_music_id  = -1,
    main_music_id   = -1,
    ready_id        = -1,
    engines_id      = -1,
    scream_id       = -1,
    game_over_id    = -1,
    mine_powerup_id = -1,
    deactivate_id   = -1,
    main_menu_id    = -1,
    empty_id        = -1,
    beep0_id             = -1,
    beep1_id             = -1,
    station_blow_id      = -1,
    station_throb_id     = -1,
    ammopu_id            = -1,
    shldpu_id            = -1;
*/
int expl_ids[MAX_EXPL_SOUNDS] = {-1,-1,-1,-1,-1,-1,-1,-1};
int fire_ids[MAX_FIRE_SOUNDS] = {-1,-1,-1,-1,-1,-1,-1,-1};
int game_state                = GAME_STATE_INIT;   // initial game state

// TYPES //////////////////////////////////////////////////


//! Set to true if TapWave support is detected.  This means that those APIs
//! can be used to widen the screen and read the joystick.
static Boolean TapWaveSupport;

//! Array of input information the app is interested in.  Other values 
//! like buttons could also be included here.
//!
//! NOTE: This structure must be four bytes aligned because it is used by ARM based
//! API.
//static Int32 InputFormats[] = {twInputNavX, twInputNavY};
static Int32 InputFormats[] = {twInputNavX, twInputNavY, twInputActionA, twInputActionB, 
							   twInputActionC, twInputActionD, twInputTriggerA, twInputTriggerB};


//static const RGBColorType ColorGameMenu = { 0, 239, 211, 16 };
static TwGfxPackedRGBType ColorGameMenu;


static Int32 GTimeOut = 3000;
MemHandle MusicH;
MemPtr	  MusicP;
TwSmfHandle mMusic;
TwMidiHandle mDummy;
MemHandle mMusicHandle;

/*! \brief  Mask the keys to reduce keyDownEvents from being sent.

This app does not use the events but polls the keys instead.  This also
sets up the joystick for analog polling.

\see HostUnmaskKeys

*/

//AK temp storage for scaled bitmaps
// This is the offscreen scratch buffer
TwGfxSurfaceType * TempBufferH;    // screenBuffer
UInt8 * TempBufferP;  // DOLATER - This is really a pointer to the pixels!

asm char TwGfxOpenNoSignV2 (void *p, void *q)
{
	stmfd sp!, {r4-r11,lr}
	ldr r9, [r9]
	ldr r9, [r9]
	sub sp, sp, #0x24
	mov r6, r0
	mov r7, r1
	ldr pc, =0x200995F0
} 


void HostDrawParticle(int x, int y, int colour)
{
	TwGfxPointType points[1];
	Err err;

	points->x = x;
	points->y = y;

        // Copy the source window (contains the image to draw) to the draw window.
        err = TwGfxDrawPoints(GameApp.screenBufferH, points, 1, RGBColour_range[colour]);
	    ErrNonFatalDisplayIf(err != errNone, "TwGfxBitBlt problem");


}
void HostDrawPixel(int x, int y, int colour)
{
	TwGfxPointType points[1];
	Err err;

	points->x = x;
	points->y = y;

        // Copy the source window (contains the image to draw) to the draw window.
        err = TwGfxDrawPoints(GameApp.screenBufferH, points, 1, RGBColour_range[colour]);
	    ErrNonFatalDisplayIf(err != errNone, "TwGfxBitBlt problem");


}
void HostDrawPixel16(int x, int y, TwGfxPackedRGBType RGBcolour)
{
	TwGfxPointType points[1];
	Err err;

	points->x = x;
	points->y = y;

        // Copy the source window (contains the image to draw) to the draw window.
        err = TwGfxDrawPoints(GameApp.screenBufferH, points, 1, RGBcolour);
	    ErrNonFatalDisplayIf(err != errNone, "TwGfxBitBlt problem");


}

void HostDrawParticles(int count)
{
	Err err;


        // Copy the source window (contains the image to draw) to the draw window.
        err = TwGfxDrawColorPoints(GameApp.screenBufferH, (TwGfxPointType *)hostparticles, count, hostparticles_colour);
	    ErrNonFatalDisplayIf(err != errNone, "TwGfxBitBlt problem");


}

void PlayToneOS5(int EffectID)
{
	MemHandle h;
	h = DmGetResource('wave', EffectID);  // find the resource in this PRC (added in the .r file)
	if (h) {
		// play it at the standard volume for game sounds, synchronously.
		SndPlayResource(MemHandleLock(h), sndGameVolume, sndFlagAsync);
		MemHandleUnlock(h);
	}
}

void PlayMidiMusic(UInt16 MusicID)
{
	Err error;

    //AK Music handle
    MusicH = DmGetResource('MIDI',MusicID);
    MusicP = MemHandleLock(MusicH);
	// AK start music playing
	error = TwMidiPlaySmf(MusicP,&MusicDuration,true);

}
UInt8 CheckMidiDuration()
{

	return MusicDuration;
}
void startMusic(bool aRestart) {
	Int32 track;


    if (aRestart) {
        stopMusic();
    }
    else if (mMusic) {
        return;
    }
 
    track = 3000 + (rand() % 2);
  

    TwMidiSetMasterVolume(twMidiMaxVolume / 2);

    mMusicHandle = DmGet1Resource('MIDI', (UInt16) track);
    if (mMusicHandle) {
        UInt8* smfData = (UInt8*) MemHandleLock(mMusicHandle);

        Err err = TwSmfOpen(&mMusic, smfData, NULL);
        if (err) {
            MemHandleUnlock(mMusicHandle);
            mMusicHandle = NULL;
        }
        else {
            TwMidiOpen(&mDummy);
            smfCallback(0);
        }
    }
}

void stopMusic() {
    if (mMusic) {
        TwSmfClose(mMusic);
        TwMidiClose(mDummy);
        MemHandleUnlock(mMusicHandle);
        mMusic = NULL;
        mMusicHandle = NULL;
    }
}

bool CheckMusicPlaying()
{
	bool ret;

	TwSmfIsPlaying(mMusic,&ret);
	
	return ret;
}

void smfCallback(UInt32 aUserData) {
    
    TwSmfPlay(mMusic, NULL, NULL, NULL);
}

void Draw_Scanner(void)
{
// this function draws the scanner

int index,sx,sy; // looping and position
int col=0;
int count = 0;
// lock back surface

// draw all the rocks
for (index=0; index < MAX_ROCKS; index++)
    {
    // draw rock blips
	if (rocks[index].state==ROCK_STATE_ON)
		{
		sx = ((rocks[index].varsI[INDEX_WORLD_X] - UNIVERSE_MIN_X) >> 7) + (SCREEN_WIDTH/2) - ((UNIVERSE_MAX_X - UNIVERSE_MIN_X) >> 8);
		sy = ((rocks[index].varsI[INDEX_WORLD_Y] - UNIVERSE_MIN_Y) >> 7) + 5;
	    
		//HostDrawPixel16(sx,sy,TwGfxComponentsToPackedRGB(128,128,128));
		hostparticles[count].x = sx;
		hostparticles[count].y = sy;
		hostparticles_colour[count].Colour = TwGfxComponentsToPackedRGB(128,128,128);
		++count;

		} // end if

    } // end for index


// draw all the gunships
for (index=0; index < MAX_GUNSHIPS; index++)
    {
    // draw gunship blips
	if (gunships[index].state==GUNSHIP_STATE_ALIVE)
		{
		sx = ((gunships[index].varsI[INDEX_WORLD_X] - UNIVERSE_MIN_X) >> 7) + (SCREEN_WIDTH/2) - ((UNIVERSE_MAX_X - UNIVERSE_MIN_X) >> 8);
		sy = ((gunships[index].varsI[INDEX_WORLD_Y] - UNIVERSE_MIN_Y) >> 7) + 5;
	    
		//HostDrawPixel16(sx,sy,TwGfxComponentsToPackedRGB(255,255,0));
		//HostDrawPixel16(sx+1,sy,TwGfxComponentsToPackedRGB(255,255,0));
		hostparticles[count].x = sx;
		hostparticles[count].y = sy;
		hostparticles_colour[count].Colour = TwGfxComponentsToPackedRGB(255,255,0);
		++count;
		hostparticles[count].x = sx;
		hostparticles[count].y = sy;
		hostparticles_colour[count].Colour = TwGfxComponentsToPackedRGB(255,255,0);
		++count;
		} // end if

    } // end for index

// draw all the mines
for (index=0; index < MAX_MINES; index++)
    {
    // draw gunship blips
	if (mines[index].state==MINE_STATE_ALIVE)
		{
		sx = ((mines[index].varsI[INDEX_WORLD_X] - UNIVERSE_MIN_X) >> 7) + (SCREEN_WIDTH/2) - ((UNIVERSE_MAX_X - UNIVERSE_MIN_X) >> 8);
		sy = ((mines[index].varsI[INDEX_WORLD_Y] - UNIVERSE_MIN_Y) >> 7) + 5;
	    
		//HostDrawPixel16(sx,sy,TwGfxComponentsToPackedRGB(255,0,0));
		//HostDrawPixel16(sx,sy+1,TwGfxComponentsToPackedRGB(255,0,0));
		hostparticles[count].x = sx;
		hostparticles[count].y = sy;
		hostparticles_colour[count].Colour = TwGfxComponentsToPackedRGB(255,0,0);
		++count;
		hostparticles[count].x = sx;
		hostparticles[count].y = sy;
		hostparticles_colour[count].Colour = TwGfxComponentsToPackedRGB(255,0,0);
		++count;
		} // end if

    } // end for index

// unlock the secondary surface

// draw all the stations
for (index=0; index < MAX_STATIONS; index++)
    {
    // draw station blips
	if (stations[index].state==STATION_STATE_ALIVE)
		{
		sx = ((stations[index].varsI[INDEX_WORLD_X] - UNIVERSE_MIN_X) >> 7) + (SCREEN_WIDTH/2) - ((UNIVERSE_MAX_X - UNIVERSE_MIN_X) >> 8);
		sy = ((stations[index].varsI[INDEX_WORLD_Y] - UNIVERSE_MIN_Y) >> 7) + 5;
	    
        // test for state
		if (stations[index].anim_state == STATION_SHIELDS_ANIM_ON)
			{
            stationsmall.curr_frame = 0;
            stationsmall.x = sx - 3;
            stationsmall.y = sy - 3; 
            NewDraw_BOB(&stationsmall);            


			} // end if
		else
			{

            stationsmall.curr_frame = 1;
            stationsmall.x = sx - 3;
            stationsmall.y = sy - 3; 
            NewDraw_BOB(&stationsmall);  


			} // end if

		} // end if

    } // end for index


// unlock the secondary surface

// draw player as white blip
sx = (((int)(player_x) - UNIVERSE_MIN_X) >> 7) + (SCREEN_WIDTH/2) - ((UNIVERSE_MAX_X - UNIVERSE_MIN_X) >> 8);
sy = (((int)(player_y) - UNIVERSE_MIN_Y) >> 7) + 5;

col = rand()%256;

//HostDrawPixel(sx,sy,col);
//HostDrawPixel(sx+1,sy,col);
//HostDrawPixel(sx,sy+1,col);
//HostDrawPixel(sx+1,sy+1,col);
		hostparticles[count].x = sx;
		hostparticles[count].y = sy;
		hostparticles_colour[count].Colour = RGBColour_range[col];
		++count;
		hostparticles[count].x = sx+1;
		hostparticles[count].y = sy;
		hostparticles_colour[count].Colour = RGBColour_range[col];
		++count;
		hostparticles[count].x = sx;
		hostparticles[count].y = sy+1;
		hostparticles_colour[count].Colour = RGBColour_range[col];
		++count;
		hostparticles[count].x = sx+1;
		hostparticles[count].y = sy+1;
		hostparticles_colour[count].Colour = RGBColour_range[col];
		++count;

// unlock the secondary surface

// now draw the art around the edges

hud.x          = (SCREEN_WIDTH/2)-64;
hud.y          = 5-4;
hud.curr_frame = 0;
NewDraw_BOB(&hud);

hud.x          = (SCREEN_WIDTH/2)+64-16;
hud.y          = 5-4;
hud.curr_frame = 1;
NewDraw_BOB(&hud);

hud.x          = (SCREEN_WIDTH/2)-64;
hud.y          = 5+128-20;
hud.curr_frame = 2;
NewDraw_BOB(&hud);


hud.x          = (SCREEN_WIDTH/2)+64-16;
hud.y          = 5+128-20;
hud.curr_frame = 3;
NewDraw_BOB(&hud);

HostDrawParticles(count);

} // end Draw_Scanner

void Init_Stations(void)
{
// this function loads and initializes the stations to a known state
	
static int shields_on_anim[16] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15},
           shields_off_anim[1] = {16};

int frame;  // looping va
int ship;
// create the first bob
Create_BOB(&stations[0],0,0,192,144,17,0,BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_ANIM | BOB_ATTR_ABS);
Set_Transparent_BOB(&stations,true,0,0,0);

// load animation frames 
for (frame=0; frame <= 16; frame++)
	{
	// load the station imagery 
    //Pad_Name("OUTART/STATION", "BMP", buffer, frame);
	//Load_Bitmap_File(&bitmap8bit, buffer);		

    // load the actual .BMP
    Load_Frame_BOB(&stations[0],Station0 + frame,frame,0,0,BITMAP_EXTRACT_MODE_ABS);  

	} // end if

// set state to off
stations[0].state  = STATION_STATE_ALIVE;

// set anim state
stations[0].anim_state = STATION_SHIELDS_ANIM_ON;

// set damage to 0
stations[0].varsI[INDEX_STATION_DAMAGE] = 0;

// set animation rate
Set_Anim_Speed_BOB(&stations[0],15);

// load in the shield on/off animations
Load_Animation_BOB(&stations[0], STATION_SHIELDS_ANIM_ON, 16, shields_on_anim);
Load_Animation_BOB(&stations[0], STATION_SHIELDS_ANIM_OFF, 1, shields_off_anim);

// set animation to on
Set_Animation_BOB(&stations[0], STATION_SHIELDS_ANIM_ON);

// make copies
for (ship=1; ship < MAX_STATIONS; ship++)
    {
		//MemMove(&stations[ship], &stations[0], sizeof(BOB));
		Clone_BOB(&stations[0],&stations[ship]);
    } // end for pulse


    // load the miniature station
    Create_BOB(&stationsmall,0,0,8,8,2,2,BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME | BOB_ATTR_CELL);
	Set_Transparent_BOB(&stationsmall,true,0,0,0);

    //Load_Bitmap_File(&bitmap8bit, "OUTART/STATIONSMALL8.BMP");		

    // load the actual .BMP
    Load_Frame_BOB(&stationsmall,StationSmall8,0,0,0,BITMAP_EXTRACT_MODE_ABS);  
    //Load_Frame_BOB(&stationsmall,StationSmall8_2,1,1,0,BITMAP_EXTRACT_MODE_CELL);  

    // unload data infile
    //Unload_Bitmap_File(&bitmap8bit);


} // end Init_Stations

///////////////////////////////////////////////////////////

void Reset_Stations(void)
{
	int index;
// this function resets all the stations in preparation for another run

for (index=0; index < MAX_STATIONS; index++)
	{
    // set state to off
    stations[index].state  = STATION_STATE_DEAD;

    // set anim state
    stations[index].anim_state = STATION_SHIELDS_ANIM_ON;

    // set damage to 0
    stations[index].varsI[INDEX_STATION_DAMAGE] = 0;

    } // end for

} // end Reset_Stations

///////////////////////////////////////////////////////////

void Start_Station(int override, int x, int y)
{
	int index;
	int xpos,ypos,ang;
	float mine_x,mine_y;
// this functions starts a station, note that if override = 1
// then the function uses the sent data otherwise it's random

// first find an available stations
for (index=0; index < MAX_STATIONS; index++)
	{
	// is this one dead
	if (stations[index].state  == STATION_STATE_DEAD)
		{
		// position the station

		xpos = RAND_RANGE((UNIVERSE_MIN_X+256),(UNIVERSE_MAX_X-256));
		ypos = RAND_RANGE((UNIVERSE_MIN_Y+256),(UNIVERSE_MAX_Y-256));

        // set position
		stations[index].varsI[INDEX_WORLD_X] = xpos;
		stations[index].varsI[INDEX_WORLD_Y] = ypos;

        // start a mine up in the vicinity
        ang = rand()%16;
		mine_x = xpos + STATION_RANGE_RING*cos_look16[ang];
		mine_y = ypos + STATION_RANGE_RING*sin_look16[ang];

        // start a deactivated mine
        Start_Mine(1, mine_x, mine_y, MINE_STATE_AI_SLEEP);    

  	    // set velocity
		stations[index].xv = 0;
		stations[index].yv = 0;

		// set remaining state variables
		stations[index].state  = STATION_STATE_ALIVE;

		// set animation to on
        Set_Animation_BOB(&stations[index], STATION_SHIELDS_ANIM_ON);

		// set anim state
        stations[index].anim_state = STATION_SHIELDS_ANIM_ON;

        // set damage to 0
        stations[index].varsI[INDEX_STATION_DAMAGE] = 0;

		// done so exit
		return;

		} // end if
	
	} // end for index

} // end Start_Station

////////////////////////////////////////////////////////////

void Move_Stations(void)
{
	int index;
// this function moves/animates all the stations

for (index=0; index < MAX_STATIONS; index++)
    {
    // test if station is alive
    if (stations[index].state == STATION_STATE_ALIVE)
        {
        // move the stations
        stations[index].varsI[INDEX_WORLD_X]+=stations[index].xv;
        stations[index].varsI[INDEX_WORLD_Y]+=stations[index].yv;
          
        // test for boundaries
        if (stations[index].varsI[INDEX_WORLD_X] > UNIVERSE_MAX_X)
           {
  		   stations[index].varsI[INDEX_WORLD_X] = UNIVERSE_MIN_X;
           } // end if
		else
		if (stations[index].varsI[INDEX_WORLD_X] < UNIVERSE_MIN_X )
           {
  		   stations[index].varsI[INDEX_WORLD_X] = UNIVERSE_MAX_X;
           } // end if	    

		if (stations[index].varsI[INDEX_WORLD_Y] > UNIVERSE_MAX_Y)
           {
		   stations[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MIN_Y;
           } // end if
		else
		if (stations[index].varsI[INDEX_WORLD_Y] < UNIVERSE_MIN_Y ) 
           {
		   stations[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MAX_Y;
           } // end if

        // test for damage level
        if (stations[index].varsI[INDEX_STATION_DAMAGE] > (MAX_STATION_DAMAGE/4) &&
            (rand()%(20 - (stations[index].varsI[INDEX_STATION_DAMAGE] >> 3))) == 1)
            {
            int width = 20+rand()%60;

             // start a burst
             Start_Burst(stations[index].varsI[INDEX_WORLD_X] - (stations[index].width*.5)+RAND_RANGE(0,stations[index].width),
                         stations[index].varsI[INDEX_WORLD_Y] - (stations[index].height*.5)+RAND_RANGE(0,stations[index].height), 
                         width,(width >> 2) + (width >> 1),
                         (int)(stations[index].xv)>>1, (int)(stations[index].yv)>>1);   

            // add some particles

            } // end if

		} // end if alive

	} // end for index

} // end Move_Stations

///////////////////////////////////////////////////////////

void Draw_Stations(void)
{
	int index;
// this function draws all the stations

for (index=0; index < MAX_STATIONS; index++)
    {
    // test if station is alive
    if (stations[index].state == STATION_STATE_ALIVE)
        {
        // transform to screen coords
        stations[index].x = stations[index].varsI[INDEX_WORLD_X] - (stations[index].width >> 1) - player_x + (SCREEN_WIDTH/2);
        stations[index].y = stations[index].varsI[INDEX_WORLD_Y] - (stations[index].height >> 1) - player_y + (SCREEN_HEIGHT/2);

        // draw the station
        NewDraw_BOB(&stations[index]);

		// animate the station
        Animate_BOB(&stations[index]);
         
        } // end if

    } // end for index

} // end Draw_Stations

////////////////////////////////////////////////////////////

void Delete_Stations(void)
{
	int index;
// this function simply deletes all memory and surfaces
// related to the gunships

for (index=0; index < MAX_STATIONS; index++)
    Destroy_BOB(&stations[index]);
    
Destroy_BOB(&stationsmall);
} // end Delete_Stations

///////////////////////////////////////////////////////////

void Init_Mines(void)
{
// this function loads and initializes the mines to a known state
	
int frame;  // looping va
int mine;
// create the first bob
Create_BOB(&mines[0],0,0,48,36,16,0,BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME | BOB_ATTR_ABS);
Set_Transparent_BOB(&mines,true,0,0,0);

// load animation frames 
for (frame = 0; frame <= 15; frame++)
	{
	// load the mine
    //Pad_Name("OUTART/PREDMINE", "BMP", buffer, frame);
	//Load_Bitmap_File(&bitmap8bit, buffer);		

    // load the actual .BMP
    Load_Frame_BOB(&mines[0],Mine0 + frame,frame,0,0,BITMAP_EXTRACT_MODE_ABS);  

    // unload data infile
    //Unload_Bitmap_File(&bitmap8bit);

	} // end if

// set state to off
mines[0].state  = MINE_STATE_DEAD;

// set ai state to sleep mode
mines[0].varsI[INDEX_MINE_AI_STATE] = MINE_STATE_AI_SLEEP;

// set damage to 0
mines[0].varsI[INDEX_MINE_DAMAGE] = 0;

// set contact count to 0
mines[0].varsI[INDEX_MINE_CONTACT_COUNT] = 0;

// set animation rate
Set_Anim_Speed_BOB(&mines[0],3);

// make copies
for (mine=1; mine < MAX_MINES; mine++)
    {
		//MemMove(&mines[mine], &mines[0], sizeof(BOB));
		Clone_BOB(&mines[0],&mines[mine]);
    } // end for mine

} // end Init_Mines

////////////////////////////////////////////////////////////

void Reset_Mines(void)
{
	int mine;
// this functions resets all the mines

// make copies
for (mine = 0; mine < MAX_MINES; mine++)
    {
    // set state to off
    mines[mine].state = MINE_STATE_DEAD;

    // set ai state to sleep mode
    mines[mine].varsI[INDEX_MINE_AI_STATE] = MINE_STATE_AI_SLEEP;

    // set damage to 0
    mines[mine].varsI[INDEX_MINE_DAMAGE] = 0;

    // set contact count to 0
    mines[mine].varsI[INDEX_MINE_CONTACT_COUNT] = 0;

    } // end for mine

} // end Reset_Mines

///////////////////////////////////////////////////////////

void Start_Mine(int override, int x, int y, int ai_state)
{
	int index;
// this functions starts a mine, note that if override = 1
// then the function uses the sent data otherwise it's random

// first find an available stations
for (index=0; index < MAX_MINES; index++)
	{
	// is this one dead
	if (mines[index].state  == MINE_STATE_DEAD)
		{

        if (!override)
            {
	    	// position the mine
    		int xpos = RAND_RANGE((UNIVERSE_MIN_X+256),(UNIVERSE_MAX_X-256));
		    int ypos = RAND_RANGE((UNIVERSE_MIN_Y+256),(UNIVERSE_MAX_Y-256));

            // set position
		    mines[index].varsI[INDEX_WORLD_X] = xpos;
		    mines[index].varsI[INDEX_WORLD_Y] = ypos;

  	        // set velocity
		    mines[index].xv = RAND_RANGE(-8,8);
		    mines[index].yv = RAND_RANGE(-8,8);

		    // set remaining state variables
		    mines[index].state  = MINE_STATE_ALIVE;

		    // set damage to 0
            mines[index].varsI[INDEX_MINE_DAMAGE] = 0;

            // set ai state
            mines[index].varsI[INDEX_MINE_AI_STATE] = ai_state;

            // set contact count
            mines[index].varsI[INDEX_MINE_CONTACT_COUNT] = 0;
            
		    // done so exit
		    return;

            }
        else
            {
            // set position
		    mines[index].varsI[INDEX_WORLD_X] = x;
		    mines[index].varsI[INDEX_WORLD_Y] = y;

      	    // set velocity
	    	mines[index].xv = 0;
		    mines[index].yv = 0;

		    // set remaining state variables
		    mines[index].state  = MINE_STATE_ALIVE;

		    // set damage to 0
            mines[index].varsI[INDEX_MINE_DAMAGE] = 0;

            // set ai state
            mines[index].varsI[INDEX_MINE_AI_STATE] = ai_state;

            // set contact count
            mines[index].varsI[INDEX_MINE_CONTACT_COUNT] = 0;

		    // done so exit
		    return;

            } // end else


		} // end if alive
	
	} // end for index

} // end Start_Mine

////////////////////////////////////////////////////////////

void Move_Mines(void)
{
	int index;
	float vx,vy,length;
	int width;
// this function moves/animates all the mines

for (index=0; index < MAX_MINES; index++)
    {
    // test if station is alive
    if (mines[index].state == MINE_STATE_ALIVE)
        {

// test for activation
if (mines[index].varsI[INDEX_MINE_AI_STATE] == MINE_STATE_AI_ACTIVATED)
    {

        // move the mines
        mines[index].varsI[INDEX_WORLD_X]+=mines[index].xv;
        mines[index].varsI[INDEX_WORLD_Y]+=mines[index].yv;


       // add damage trails
       if (mines[index].varsI[INDEX_MINE_DAMAGE] > (MAX_MINE_DAMAGE >> 1) && 
           (rand()%3)==1)
        {
        Start_Particle(PARTICLE_TYPE_FLICKER, PARTICLE_COLOR_WHITE, 30+rand()%25, 
                      mines[index].varsI[INDEX_WORLD_X]+RAND_RANGE(-4,4), 
                      mines[index].varsI[INDEX_WORLD_Y]+RAND_RANGE(-4,4), 
                      (mines[index].xv*.125), (mines[index].yv*.125));
    
        Start_Particle(PARTICLE_TYPE_FADE, PARTICLE_COLOR_RED, 5, 
                      mines[index].varsI[INDEX_WORLD_X]+RAND_RANGE(-4,4), 
                      mines[index].varsI[INDEX_WORLD_Y]+RAND_RANGE(-4,4), 
                      (mines[index].xv*.125), (mines[index].yv*.125));

        } // end if

        // tracking algorithm

        // compute vector toward player
        vx = player_x - mines[index].varsI[INDEX_WORLD_X];
        vy = player_y - mines[index].varsI[INDEX_WORLD_Y];

        // normalize vector (sorta :)
        length = Fast_Distance_2D(vx,vy);


        // only track if reasonable close
        if (length < MIN_MINE_TRACKING_DIST)
            {
            vx=mine_tracking_rate*vx/length;
            vy=mine_tracking_rate*vy/length;

            // add velocity vector to current velocity
            mines[index].xv+=vx;
            mines[index].yv+=vy;

            // add a little noise
            if ((rand()%10)==1)
                {
                vx = RAND_RANGE(-1,1);
                vy = RAND_RANGE(-1,1);
                mines[index].xv+=vx;
                mines[index].yv+=vy;
                } // end if

            // test velocity vector of mines
            length = Fast_Distance_2D(mines[index].xv, mines[index].yv);

            // test for velocity overflow and slow
            if (length > MAX_MINE_VELOCITY)
                {
                // slow down
                mines[index].xv*=0.75;
                mines[index].yv*=0.75;

                } // end if

            } // end if
        else
            {
            // add a random velocity component
            if ((rand()%30)==1)
                {
                vx = RAND_RANGE(-2,2);
                vy = RAND_RANGE(-2,2);

                // add velocity vector to current velocity
                mines[index].xv+=vx;
                mines[index].yv+=vy;

                // test velocity vector of mines
                length = Fast_Distance_2D(mines[index].xv, mines[index].yv);

                // test for velocity overflow and slow
                if (length > MAX_MINE_VELOCITY)
                    {
                    // slow down
                    mines[index].xv*=0.75;
                    mines[index].yv*=0.75;

                    } // end if
        
                } // end if
           
            } // end else

        } // end if activated
    else
        {
        // compute dist from player
        vx = player_x - mines[index].varsI[INDEX_WORLD_X];
        vy = player_y - mines[index].varsI[INDEX_WORLD_Y];

        length = Fast_Distance_2D(vx,vy);

        if (length < MIN_MINE_ACTIVATION_DIST)
            {
            mines[index].varsI[INDEX_MINE_AI_STATE] = MINE_STATE_AI_ACTIVATED;

            // sound off
            PlayToneOS5(MinePower1);
            } // end if

        } // end else

        // test for boundaries
        if (mines[index].varsI[INDEX_WORLD_X] > UNIVERSE_MAX_X)
           {
  		   mines[index].varsI[INDEX_WORLD_X] = UNIVERSE_MIN_X;
           } // end if
		else
		if (mines[index].varsI[INDEX_WORLD_X] < UNIVERSE_MIN_X )
           {
  		   mines[index].varsI[INDEX_WORLD_X] = UNIVERSE_MAX_X;
           } // end if	    

		if (mines[index].varsI[INDEX_WORLD_Y] > UNIVERSE_MAX_Y)
           {
		   mines[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MIN_Y;
           } // end if
		else
		if (mines[index].varsI[INDEX_WORLD_Y] < UNIVERSE_MIN_Y ) 
           {
		   mines[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MAX_Y;
           } // end if

    // check for collision with player
         if (player_state == PLAYER_STATE_ALIVE && player_state == PLAYER_STATE_ALIVE &&
             Collision_Test(player_x-(wraith.width*.5), 
                            player_y-(wraith.height*.5), 
                            wraith.width, wraith.height,
                            mines[index].varsI[INDEX_WORLD_X]-(mines[index].width*.5), 
			     		    mines[index].varsI[INDEX_WORLD_Y]-(mines[index].height*.5),
                            mines[index].width, mines[index].height)) 
             {

             // test for contact count
             if (++mines[index].varsI[INDEX_MINE_CONTACT_COUNT] > MAX_MINE_CONTACT_COUNT)
                {
            
                 // add to players score
                 player_score+=250;

                 // damage player
                 player_damage+=30;

                 // engage shields
                 player_shield_count = 3;

                 width = 30+rand()%40;

                 // start a burst
                 Start_Burst(mines[index].varsI[INDEX_WORLD_X], 
                             mines[index].varsI[INDEX_WORLD_Y], 
                             width, (width*.5) + (width*.25),
                             (int)(mines[index].xv)>>1, (int)(mines[index].yv)>>1);   


                 // kill the mine
                 mines[index].state=MINE_STATE_DEAD;

                 // start a new mine
                 Start_Mine(0,0,0,MINE_STATE_AI_ACTIVATED);
             
                // process next mine
                continue;

                } // end if contact count

             } // end if
        else // no collision or other problem, so reset
            {
            mines[index].varsI[INDEX_MINE_CONTACT_COUNT] = 0;
            } // end else

		} // end if alive

	} // end for index

} // end Move_Mines

///////////////////////////////////////////////////////////

void Draw_Mines(void)
{
	int index;
// this function draws all the mines

for (index=0; index < MAX_MINES; index++)
    {
    // test if mine is alive
    if (mines[index].state == MINE_STATE_ALIVE)
        {
        // transform to screen coords
        mines[index].x = mines[index].varsI[INDEX_WORLD_X] - (mines[index].width >> 1) - player_x + (SCREEN_WIDTH/2);
        mines[index].y = mines[index].varsI[INDEX_WORLD_Y] - (mines[index].height >> 1) - player_y + (SCREEN_HEIGHT/2);

        // draw the station
        NewDraw_BOB(&mines[index]);

		// animate the mine
        Animate_BOB(&mines[index]);
        
        } // end if

    } // end for index

} // end Draw_Mines

////////////////////////////////////////////////////////////

void Delete_Mines(void)
{
	int index;
// this function simply deletes all memory and surfaces
// related to the mines

for (index=0; index < MAX_MINES; index++)
    Destroy_BOB(&mines[index]);

} // end Delete_Mines

///////////////////////////////////////////////////////////

void Init_Gunships(void)
{
// this function loads and initializes the gunships to a known state
	
int frame;  // looping var
int ship;
// create the first bob
Create_BOB(&gunships[0],0,0,160,70,32,0,BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME | BOB_ATTR_ABS);
Set_Transparent_BOB(&gunships,true,0,0,0);

// load animation frames right
for (frame=0; frame <= 15; frame++)
	{
	// load the rocks imagery 
    //Pad_Name("OUTART/GUNSHIPR", "BMP", buffer, frame);
	//Load_Bitmap_File(&bitmap8bit, buffer);		

    // load the actual .BMP
    Load_Frame_BOB(&gunships[0],GunshipR0 + frame,frame,0,0,BITMAP_EXTRACT_MODE_ABS);  

    // unload data infile
    //Unload_Bitmap_File(&bitmap8bit);

	} // end if

// load animation frames left
for (frame=0; frame <= 15; frame++)
	{
	// load the rocks imagery 
    //Pad_Name("OUTART/GUNSHIPL", "BMP", buffer, frame);
	//Load_Bitmap_File(&bitmap8bit, buffer);		

    // load the actual .BMP
    Load_Frame_BOB(&gunships[0],GunshipL0 + frame,frame+16,0,0,BITMAP_EXTRACT_MODE_ABS);  

    // unload data infile
    //Unload_Bitmap_File(&bitmap8bit);

	} // end if

// set state to off
gunships[0].state                       = GUNSHIP_STATE_DEAD;
gunships[0].varsI[INDEX_GUNSHIP_DIR]    = GUNSHIP_RIGHT;
gunships[0].varsI[INDEX_GUNSHIP_TURRET] = 0;
gunships[0].varsI[INDEX_GUNSHIP_DAMAGE] = 0;

// make copies
for (ship=1; ship < MAX_GUNSHIPS; ship++)
    {
    MemMove(&gunships[ship], &gunships[0], sizeof(BOB));
    } // end for gunship

} // end Init_Gunships

//////////////////////////////////////////////////////////////////////////////////////

void Reset_Gunships(void)
{
	int ship;
// this function resets all the gunships

for (ship=0; ship < MAX_GUNSHIPS; ship++)
    {
    gunships[ship].state                       = GUNSHIP_STATE_DEAD;
    gunships[ship].varsI[INDEX_GUNSHIP_DIR]    = GUNSHIP_RIGHT;
    gunships[ship].varsI[INDEX_GUNSHIP_TURRET] = 0;
    gunships[ship].varsI[INDEX_GUNSHIP_DAMAGE] = 0;
    } // end for gunship

} // end Reset_Gunships

///////////////////////////////////////////////////////////

void Start_Gunship(int override, int x, int y, int dir)
{
	int index,ang;
	float xpos,ypos;
// this functions starts a gunship, note that if override = 1
// then the function uses the sent data otherwise it's random

// first find an available gunship
for (index=0; index < MAX_GUNSHIPS; index++)
	{
	// is this one dead
	if (gunships[index].state  == GUNSHIP_STATE_DEAD)
		{
		// position the gunship on a circle ring randomly from player

		ang = rand()%16;
		xpos = GUNSHIP_RANGE_RING*cos_look16[ang];
		ypos = GUNSHIP_RANGE_RING*sin_look16[ang];

        // set position
		gunships[index].varsI[INDEX_WORLD_X] = player_x+xpos;
		gunships[index].varsI[INDEX_WORLD_Y] = player_y+ypos;

		// set direction
		if ((rand()%2) == 0)
			{
			// set direction to right
			gunships[index].varsI[INDEX_GUNSHIP_DIR] = GUNSHIP_RIGHT;

			// set velocity
			gunships[index].xv = 4+rand()%8;
			gunships[index].yv = RAND_RANGE(-4,4);

			} // end if right
		else 
			{
			// set direction to left
			gunships[index].varsI[INDEX_GUNSHIP_DIR] = GUNSHIP_LEFT;

			// set velocity
			gunships[index].xv = -4-rand()%8;
			gunships[index].yv = RAND_RANGE(-4,4);

			} // end left
		
		// set remaining state variables
		gunships[index].state  = GUNSHIP_STATE_ALIVE;
        gunships[index].varsI[INDEX_GUNSHIP_TURRET] = 0;
        gunships[index].varsI[INDEX_GUNSHIP_DAMAGE] = 0;

		// done so exit
		return;

		} // end if
	
	} // end for index

} // end Start_Gunship

////////////////////////////////////////////////////////////

void Move_Gunships(void)
{
// this function moves all the gunships and does the ai

// turrent center offsets for plasma pulses
static int turret_left[3][2]  = {-60,-19, -63,-9, -3,-24};
static int turret_right[3][2] = { 60,-19,  63,-9,  3,-24};
int index;
int tdir1,tdir2;
float d1x,d1y;
float d2x,d2y;
float d0x,d0y;
float dist0,dist1,dist2;
int tdir0;
int turret;
int plasma_x,plasma_y;
float plasma_xv,plasma_yv;

for (index=0; index < MAX_GUNSHIPS; index++)
    {
    // test if plasma pulse is in flight
    if (gunships[index].state == GUNSHIP_STATE_ALIVE)
        {
        // move the gunship
        gunships[index].varsI[INDEX_WORLD_X]+=gunships[index].xv;
        gunships[index].varsI[INDEX_WORLD_Y]+=gunships[index].yv;
          
        // test for boundaries
        if (gunships[index].varsI[INDEX_WORLD_X] > UNIVERSE_MAX_X)
           {
  		   gunships[index].varsI[INDEX_WORLD_X] = UNIVERSE_MIN_X;
           } // end if
		else
		if (gunships[index].varsI[INDEX_WORLD_X] < UNIVERSE_MIN_X )
           {
  		   gunships[index].varsI[INDEX_WORLD_X] = UNIVERSE_MAX_X;
           } // end if	    

		if (gunships[index].varsI[INDEX_WORLD_Y] > UNIVERSE_MAX_Y)
           {
		   gunships[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MIN_Y;
           } // end if
		else
		if (gunships[index].varsI[INDEX_WORLD_Y] < UNIVERSE_MIN_Y ) 
           {
		   gunships[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MAX_Y;
           } // end if


       // leave particle trail
    if (gunships[index].varsI[INDEX_GUNSHIP_DAMAGE] > (MAX_GUNSHIP_DAMAGE >> 2) && 
       (rand()%3)==1)
        {
        Start_Particle(PARTICLE_TYPE_FLICKER, PARTICLE_COLOR_WHITE, 30+rand()%25, 
                      gunships[index].varsI[INDEX_WORLD_X]+RAND_RANGE(-4,4), 
                      gunships[index].varsI[INDEX_WORLD_Y]+RAND_RANGE(-4,4), 
                      (gunships[index].xv*.125), (gunships[index].yv*.125));
    
        Start_Particle(PARTICLE_TYPE_FADE, PARTICLE_COLOR_RED, 5, 
                      gunships[index].varsI[INDEX_WORLD_X]+RAND_RANGE(-4,4), 
                      gunships[index].varsI[INDEX_WORLD_Y]+RAND_RANGE(-4,4), 
                      (gunships[index].xv*.125), (gunships[index].yv*.125));


    
        } // end if

		// do ai
		
		// test if player is in range to care
		if (Fast_Distance_2D(player_x-gunships[index].varsI[INDEX_WORLD_X],
			                 player_y-gunships[index].varsI[INDEX_WORLD_Y]) < GUNSHIP_MIN_ATTACK_RANGE)
			{
            // move gunship toward player on x and y axis
                // track x-axis
                if (player_x > gunships[index].varsI[INDEX_WORLD_X])
                    gunships[index].varsI[INDEX_WORLD_X]++;
                else
                if (player_x < gunships[index].varsI[INDEX_WORLD_X])
                    gunships[index].varsI[INDEX_WORLD_X]--;
            
                // track y-axis
                if (player_y > gunships[index].varsI[INDEX_WORLD_Y])
                    gunships[index].varsI[INDEX_WORLD_Y]++;
                else
                if (player_y < gunships[index].varsI[INDEX_WORLD_Y])
                    gunships[index].varsI[INDEX_WORLD_Y]--;


            // turn turret always
            // compute d0, d1, d2
            
        

            // first create a vector point in the direction of the turret
            
            // compute current turrent vector
            tdir1 = gunships[index].varsI[INDEX_GUNSHIP_TURRET];
            
            d1x = gunships[index].varsI[INDEX_WORLD_X] + cos_look16[tdir1]*32;
            d1y = gunships[index].varsI[INDEX_WORLD_Y] + sin_look16[tdir1]*32;

            // compute turret vector plus one
            tdir2 = gunships[index].varsI[INDEX_GUNSHIP_TURRET]+1;
            
            if (tdir2 > 15) 
                tdir2 = 0;

            d2x = gunships[index].varsI[INDEX_WORLD_X] + cos_look16[tdir2]*32;
            d2y = gunships[index].varsI[INDEX_WORLD_Y] + sin_look16[tdir2]*32;
            
            // compute turret vector minus one
            tdir0 = gunships[index].varsI[INDEX_GUNSHIP_TURRET]-1;
            
            if (tdir0 < 0) 
                tdir0=15;

            d0x = gunships[index].varsI[INDEX_WORLD_X] + cos_look16[tdir0]*32;
            d0y = gunships[index].varsI[INDEX_WORLD_Y] + sin_look16[tdir0]*32;

            // now find the min dist
            dist0 = Fast_Distance_2D(player_x - d0x, player_y - d0y);
            dist1 = Fast_Distance_2D(player_x - d1x, player_y - d1y);
            dist2 = Fast_Distance_2D(player_x - d2x, player_y - d2y);

            if (dist0 < dist2 && dist0 < dist1)
                {
                // the negative direction is best        
                gunships[index].varsI[INDEX_GUNSHIP_TURRET] = tdir0;

                } // end if
            else
            if (dist2 < dist0 && dist2 < dist1)
                {
                // the positive direction is best
                gunships[index].varsI[INDEX_GUNSHIP_TURRET] = tdir2;

                } // end if

            // test for firing
            if ((rand()%25)==1)
                {   
                // which direction is ship going?
                if (gunships[index].varsI[INDEX_GUNSHIP_DIR] == GUNSHIP_LEFT)
                    {
                    // select turret to fire from
                    turret = rand()%3;

                    // compute offsets firing position
                    plasma_x = gunships[index].varsI[INDEX_WORLD_X] +  turret_left[turret][0];
                    plasma_y = gunships[index].varsI[INDEX_WORLD_Y] +  turret_left[turret][1];

                    plasma_xv = gunships[index].xv+cos_look16[gunships[index].varsI[INDEX_GUNSHIP_TURRET]]*PLASMA_SPEED_SLOW;
	                plasma_yv = gunships[index].yv+sin_look16[gunships[index].varsI[INDEX_GUNSHIP_TURRET]]*PLASMA_SPEED_SLOW;

                    // fire weapon
                    Fire_Plasma(plasma_x, plasma_y, plasma_xv,plasma_yv,PLASMA_ANIM_ENEMY);

                    } // end if
                else
                    {
                    // select turret to fire from
                    turret = rand()%3;

                    // compute offsets firing position
                    plasma_x = gunships[index].varsI[INDEX_WORLD_X] +  turret_right[turret][0];
                    plasma_y = gunships[index].varsI[INDEX_WORLD_Y] +  turret_right[turret][1];

                    plasma_xv = gunships[index].xv+cos_look16[gunships[index].varsI[INDEX_GUNSHIP_TURRET]]*PLASMA_SPEED_SLOW;
	                plasma_yv = gunships[index].yv+sin_look16[gunships[index].varsI[INDEX_GUNSHIP_TURRET]]*PLASMA_SPEED_SLOW;

                    // fire weapon
                    Fire_Plasma(plasma_x, plasma_y, plasma_xv,plasma_yv,PLASMA_ANIM_ENEMY);

                    } // end if

                 } // end if fire

			} // end if

		} // end if alive

	} // end for index

} // end Move_Gunships

///////////////////////////////////////////////////////////

void Draw_Gunships(void)
{
	int index;
// this function moves all the gunships

for (index=0; index < MAX_GUNSHIPS; index++)
    {
    // test if plasma pulse is in flight
    if (gunships[index].state == GUNSHIP_STATE_ALIVE)
        {
        // transform to screen coords
        gunships[index].x = gunships[index].varsI[INDEX_WORLD_X] - (gunships[index].width >> 1) - player_x + (SCREEN_WIDTH/2);
        gunships[index].y = gunships[index].varsI[INDEX_WORLD_Y] - (gunships[index].height >> 1) - player_y + (SCREEN_HEIGHT/2);

        // select the proper frame
		gunships[index].curr_frame = (gunships[index].varsI[INDEX_GUNSHIP_DIR] << 4) + 
			                         (gunships[index].varsI[INDEX_GUNSHIP_TURRET]);

        // draw the gunship
        NewDraw_BOB(&gunships[index]);
         
        } // end if

    } // end for index

} // end Draw_Gunships

////////////////////////////////////////////////////////////

void Delete_Gunships(void)
{
	int index;
// this function simply deletes all memory and surfaces
// related to the gunships

for (index=0; index < MAX_GUNSHIPS; index++)
    Destroy_BOB(&gunships[index]);

} // end Delete_Gunships

/////////////////////////////////////////////////////

void Init_Powerups(void)
{
// this function loads and initializes the powerups to a known state
	
int index;  // looping var

// create the first bob
Create_BOB(&powerups[0],0,0,32,32,2,0,BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME | BOB_ATTR_ABS);
Set_Transparent_BOB(&powerups,true,0,0,0);

// load the powerups 
//Pad_Name("OUTART/AMMOPU", "BMP", buffer, 0);
//Load_Bitmap_File(&bitmap8bit, buffer);		

// load the actual .BMP
Load_Frame_BOB(&powerups[0],AMMOPU,0,0,0,BITMAP_EXTRACT_MODE_ABS);  

// unload data infile
//Unload_Bitmap_File(&bitmap8bit);

//Pad_Name("OUTART/SHLDPU", "BMP", buffer, 0);
//Load_Bitmap_File(&bitmap8bit, buffer);		

// load the actual .BMP
Load_Frame_BOB(&powerups[0],SHLDPU,1,0,0,BITMAP_EXTRACT_MODE_ABS);  

// unload data infile
//Unload_Bitmap_File(&bitmap8bit);

// set state to off
powerups[0].state                     = POWERUP_STATE_DEAD;
powerups[0].varsI[INDEX_POWERUP_TYPE] = POWERUP_TYPE_AMMO;

// make copies
for (index=1; index < MAX_POWERUPS; index++)
    {
    Clone_BOB(&powerups[0], &powerups[index]);
    } // end for index

} // end Init_Powerups

//////////////////////////////////////////////////////////////////////////////////////

void Reset_Powerups(void)
{
	int index;
// this function resets all the powerups

for (index=0; index < MAX_POWERUPS; index++)
    {
    powerups[index].state                      = POWERUP_STATE_DEAD;
    powerups[index].varsI[INDEX_POWERUP_TYPE]  = POWERUP_TYPE_AMMO;
    } // end for powerups

} // end Reset_Powerups

///////////////////////////////////////////////////////////

void Start_Powerup(int type, int x, int y)
{
	int index;
// this functions starts a powerup if one is available

// first find an available powerup
for (index=0; index < MAX_POWERUPS; index++)
	{
	// is this one dead
	if (powerups[index].state == POWERUP_STATE_DEAD)
		{
		// position the powerup

        // set position
		powerups[index].varsI[INDEX_WORLD_X] = x;
		powerups[index].varsI[INDEX_WORLD_Y] = y;

        // set velocity
        powerups[index].xv = RAND_RANGE(-2,2);
        powerups[index].yv = RAND_RANGE(-2,2);

		// set remaining state variables
		powerups[index].state                     = POWERUP_STATE_ALIVE;
        powerups[index].varsI[INDEX_POWERUP_TYPE] = type;

        // set lifespan
        powerups[index].counter_1 = RAND_RANGE(100,300);

		// done so exit
		return;

		} // end if
	
	} // end for index

} // end Start_Powerup

////////////////////////////////////////////////////////////

void Move_Powerups(void)
{
	int index;
// this function moves all the powerups and does the ai

for (index=0; index < MAX_POWERUPS; index++)
    {
    // test if powerup is alive
    if (powerups[index].state == POWERUP_STATE_ALIVE)
        {
        // check for collision with player
        if (Collision_Test(player_x-(wraith.width*.5), 
                           player_y-(wraith.height*.5), 
                            wraith.width, wraith.height,
                            powerups[index].varsI[INDEX_WORLD_X]-(powerups[index].width*.5), 
			     		    powerups[index].varsI[INDEX_WORLD_Y]-(powerups[index].height*.5),
                            powerups[index].width, powerups[index].height)) 
             {
             // do the powerup
             if (powerups[index].varsI[INDEX_POWERUP_TYPE] == POWERUP_TYPE_AMMO)
                {
                // increase ammo
                player_ammo+=100;

                // make sound
                PlayToneOS5(AmmoPU);
                } // end if
             else
             if (powerups[index].varsI[INDEX_POWERUP_TYPE] == POWERUP_TYPE_SHLD)
                {
                // increase shield
                if ((player_damage-=10) < 0)
                     player_damage = 0;

                // make sound
                PlayToneOS5(ShieldPU);

                } // end if
      
             // kill powerup
             powerups[index].state = POWERUP_STATE_DEAD;
             } // end if

        // move the powerups
        powerups[index].varsI[INDEX_WORLD_X]+=powerups[index].xv;
        powerups[index].varsI[INDEX_WORLD_Y]+=powerups[index].yv;

        // test if time to terminate
        if (--powerups[index].counter_1 < 0)
           {
           // kill powerup
           powerups[index].state = POWERUP_STATE_DEAD;
           } // end if
           
        // test for boundaries
        if (powerups[index].varsI[INDEX_WORLD_X] > UNIVERSE_MAX_X)
           {
  		   powerups[index].varsI[INDEX_WORLD_X] = UNIVERSE_MIN_X;
           } // end if
		else
		if (powerups[index].varsI[INDEX_WORLD_X] < UNIVERSE_MIN_X )
           {
  		   powerups[index].varsI[INDEX_WORLD_X] = UNIVERSE_MAX_X;
           } // end if	    

		if (powerups[index].varsI[INDEX_WORLD_Y] > UNIVERSE_MAX_Y)
           {
		   powerups[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MIN_Y;
           } // end if
		else
		if (powerups[index].varsI[INDEX_WORLD_Y] < UNIVERSE_MIN_Y ) 
           {
		   powerups[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MAX_Y;
           } // end if

		} // end if alive

	} // end for index

} // end Move_Powerups

///////////////////////////////////////////////////////////

void Draw_Powerups(void)
{
	int index;
// this function draws all the Powerups

for (index=0; index < MAX_POWERUPS; index++)
    {
    // test if Powerups pulse is aluive
    if (powerups[index].state == POWERUP_STATE_ALIVE)
        {
        // transform to screen coords
        powerups[index].x = powerups[index].varsI[INDEX_WORLD_X] - (powerups[index].width >> 1) - player_x + (SCREEN_WIDTH/2);
        powerups[index].y = powerups[index].varsI[INDEX_WORLD_Y] - (powerups[index].height >> 1) - player_y + (SCREEN_HEIGHT/2);

        // select the proper frame
		powerups[index].curr_frame = powerups[index].varsI[INDEX_POWERUP_TYPE];

        // draw the Powerups
        NewDraw_BOB(&powerups[index]);
         
        } // end if

    } // end for index

} // end Draw_Powerups

////////////////////////////////////////////////////////////

void Delete_Powerups(void)
{
	int index;
// this function simply deletes all memory and surfaces
// related to the Powerups

for (index=0; index < MAX_POWERUPS; index++)
    Destroy_BOB(&powerups[index]);

} // end Delete_Powerups

//////////////////////////////////////////////////////

void Init_Plasma(void)
{
	int pulse;
// this function initializes and loads all the plasma 
// weapon pulses

// the plasma animations
static int plasma_anim_player[] = {0,1,2,3,4,5,6,7},
   		   plasma_anim_enemy[]  = {8,9,10,11,12,13,14,15};

// load the plasma imagery 
//Load_Bitmap_File(&bitmap8bit, "OUTART/ENERGY8.BMP");

// create the first bob
Create_BOB(&plasma[0],0,0,8,8,16,8,BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_ANIM | BOB_ATTR_CELL);
Set_Transparent_BOB(&plasma,true,0,0,0);

// load animation frames
//for (int frame=0; frame < 16; frame++)
     Load_Frame_BOB(&plasma[0],ENERGY8,0,0,0,BITMAP_EXTRACT_MODE_ABS);  

// set animation rate
Set_Anim_Speed_BOB(&plasma[0],1);

// load animations
Load_Animation_BOB(&plasma[0], PLASMA_ANIM_PLAYER, 8, plasma_anim_player);
Load_Animation_BOB(&plasma[0], PLASMA_ANIM_ENEMY, 8, plasma_anim_enemy);

// set state to off
plasma[0].state = PLASMA_STATE_OFF;

for (pulse=1; pulse < MAX_PLASMA; pulse++)
    {
    //MemMove(&plasma[pulse], &plasma[0], sizeof(BOB));
		Clone_BOB(&plasma[0],&plasma[pulse]);
    } // end for pulse

// unload data infile
//Unload_Bitmap_File(&bitmap8bit);

} // end Init_Plasma

/////////////////////////////////////////////////////////

void Reset_Plasma(void)
{
	int pulse;
// this function resets all the plasma pulses
for (pulse = 0; pulse < MAX_PLASMA; pulse++)
    {
    plasma[pulse].state = PLASMA_STATE_OFF;
    } // end for pulse

} // end Reset_Plasma

///////////////////////////////////////////////////////////

void Delete_Plasma(void)
{
	int index;
// this function simply deletes all memory and surfaces
// related to the plasma pulses

for (index=0; index < MAX_PLASMA; index++)
    Destroy_BOB(&plasma[index]);

} // end Delete_Plasma

///////////////////////////////////////////////////////////

void Move_Plasma(void)
{
	int index;
	int mine;
	int gunship;
	int station;
	int rock;
	int width;
// this function moves all the plasma pulses and checks for
// collision with the rocks

for (index=0; index < MAX_PLASMA; index++)
    {
    // test if plasma pulse is in flight
    if (plasma[index].state == PLASMA_STATE_ON)
        {
        // move the pulse 
        plasma[index].varsI[INDEX_WORLD_X]+=plasma[index].xv;
        plasma[index].varsI[INDEX_WORLD_Y]+=plasma[index].yv;
          
        // test for boundaries
        if ( (++plasma[index].counter_1 > PLASMA_RANGE_1)              || 
                (plasma[index].varsI[INDEX_WORLD_X] > UNIVERSE_MAX_X)  || 
                (plasma[index].varsI[INDEX_WORLD_X] < UNIVERSE_MIN_X ) ||
	            (plasma[index].varsI[INDEX_WORLD_Y] > UNIVERSE_MAX_Y)  || 
                (plasma[index].varsI[INDEX_WORLD_Y] < UNIVERSE_MIN_Y ) )
            {
            // kill the pulse
            plasma[index].state = PLASMA_STATE_OFF;
            plasma[index].counter_1 = 0;

            // move to next pulse
            continue;
            } // end if


 // test for mines /////////////////////////////////////////////////////
 
for (mine=0; mine < MAX_MINES; mine++)
     {
     if (mines[mine].state==MINE_STATE_ALIVE && 
         plasma[index].anim_state == PLASMA_ANIM_PLAYER  )
         {
         // test for collision 
         if (Collision_Test(plasma[index].varsI[INDEX_WORLD_X]-(plasma[index].width*.5), 
                            plasma[index].varsI[INDEX_WORLD_Y]-(plasma[index].height*.5), 
                            plasma[index].width, plasma[index].height,
                            mines[mine].varsI[INDEX_WORLD_X]-(mines[mine].width*.5), 
			     		    mines[mine].varsI[INDEX_WORLD_Y]-(mines[mine].height*.5),
                            mines[mine].width, mines[mine].height))
             {
             // kill pulse
             plasma[index].state     = PLASMA_STATE_OFF;
             plasma[index].counter_1 = 0;

             // do damage to mine
             mines[mine].varsI[INDEX_MINE_DAMAGE]+=(1+rand()%3);

             // increase the damage on the mine and test for death
             if (mines[mine].varsI[INDEX_MINE_DAMAGE] > MAX_MINE_DAMAGE)
                {
                // kill the ship    
                mines[mine].state=MINE_STATE_DEAD;

                // start a new mine
                Start_Mine(0,0,0,MINE_STATE_AI_ACTIVATED);

                // add to players score
                player_score+=250;

                } // end if

             width = 30+rand()%40;

             // start a burst
             Start_Burst(plasma[index].varsI[INDEX_WORLD_X], 
                         plasma[index].varsI[INDEX_WORLD_Y], 
                         width, (width*.5) + (width*.25),
                         (int)(mines[mine].xv)>>1, (int)(mines[mine].yv)>>1);   
             break; 
             } // end if
     
     } // end if alive

     } // end for mines


////////////////////////////////////////////////////

// test for gunships
 for (gunship=0; gunship < MAX_GUNSHIPS; gunship++)
     {
     if (gunships[gunship].state==GUNSHIP_STATE_ALIVE && 
         plasma[index].anim_state == PLASMA_ANIM_PLAYER  )
         {
         // test for collision 
         if (Collision_Test(plasma[index].varsI[INDEX_WORLD_X]-(plasma[index].width*.5), 
                            plasma[index].varsI[INDEX_WORLD_Y]-(plasma[index].height*.5), 
                            plasma[index].width, plasma[index].height,
                            gunships[gunship].varsI[INDEX_WORLD_X]-(gunships[gunship].width*.5), 
			     		    gunships[gunship].varsI[INDEX_WORLD_Y]-(gunships[gunship].height*.5),
                            gunships[gunship].width, gunships[gunship].height))
             {
             // kill pulse
             plasma[index].state = PLASMA_STATE_OFF;
             plasma[index].counter_1 = 0;

             // do damage to gunship
             gunships[gunship].varsI[INDEX_GUNSHIP_DAMAGE]+=(1+rand()%3);

             // increase the damage on the gunship and test for death
             if (gunships[gunship].varsI[INDEX_GUNSHIP_DAMAGE] > MAX_GUNSHIP_DAMAGE)
                {
                // kill the ship    
                gunships[gunship].state=GUNSHIP_STATE_DEAD;

                // add to players score
                player_score+=1000;

                } // end if

             // start a burst
             Start_Burst(plasma[index].varsI[INDEX_WORLD_X], 
                         plasma[index].varsI[INDEX_WORLD_Y], 
                         40+rand()%20,30+rand()%16,
                         (int)(gunships[gunship].xv)>>1, (int)(gunships[gunship].yv)>>1);   
             break; 
             } // end if
     
     } // end if alive

     } // end for gunships

///////////////////////////////////////////////////////////////////////


// test for stations
 for (station=0; station < MAX_STATIONS; station++)
     {
     if (stations[station].state == STATION_STATE_ALIVE && 
         plasma[index].anim_state == PLASMA_ANIM_PLAYER  )
         {
         // test for collision 
         if ( //stations[station].anim_state == STATION_SHIELDS_ANIM_OFF &&
             Collision_Test(plasma[index].varsI[INDEX_WORLD_X]-(plasma[index].width*.5), 
                            plasma[index].varsI[INDEX_WORLD_Y]-(plasma[index].height*.5), 
                            plasma[index].width, plasma[index].height,
                            stations[station].varsI[INDEX_WORLD_X]-(stations[station].width*.5), 
			     		    stations[station].varsI[INDEX_WORLD_Y]-(stations[station].height*.5),
                            stations[station].width, stations[station].height))
             {
             // kill pulse
             plasma[index].state = PLASMA_STATE_OFF;
             plasma[index].counter_1 = 0;

             // do damage to station
             stations[station].varsI[INDEX_STATION_DAMAGE]+=(1+rand()%3);

             // increase the damage on the station and test for death
             if (stations[station].varsI[INDEX_STATION_DAMAGE] > MAX_STATION_DAMAGE)
                {
                // kill the station
                stations[station].state=STATION_STATE_DEAD;

                // add to players score
                player_score+=10000;

                // has played won
                if (++num_stations_destroyed >= NUM_ACTIVE_STATIONS)
                   {
                   win_counter = 0;
                   player_won = 1;   
                   } // end if

                // make big sound
                PlayToneOS5(StationBlow);

                } // end if

             // start a burst
             Start_Burst(plasma[index].varsI[INDEX_WORLD_X], 
                         plasma[index].varsI[INDEX_WORLD_Y], 
                         40+rand()%20,30+rand()%16,
                         (int)(stations[station].xv)>>1, (int)(stations[station].yv)>>1);   
             break; 
             } // end if
     
     } // end if alive

     } // end for stations


///////////////////////////////////////////////////////


    // test for collision with player
    if (plasma[index].anim_state == PLASMA_ANIM_ENEMY && player_state == PLAYER_STATE_ALIVE &&
        Collision_Test(plasma[index].varsI[INDEX_WORLD_X]-(plasma[index].width*.5), 
		               plasma[index].varsI[INDEX_WORLD_Y]-(plasma[index].height*.5), 
                       plasma[index].width, plasma[index].height,
                       player_x-(wraith.width*.5), 
                       player_y-(wraith.height*.5),
                       wraith.width, wraith.height))
        {

        Start_Burst(plasma[index].varsI[INDEX_WORLD_X], 
                    plasma[index].varsI[INDEX_WORLD_Y], 
                    40+rand()%20,30+rand()%16,
                    (int)(player_xv)>>1, (int)(player_yv)>>1);

        // update players damage
        player_damage+=2;

    
        // engage shields
        player_shield_count=3;

        // kill the pulse
        plasma[index].state = PLASMA_STATE_OFF;
        plasma[index].counter_1 = 0;

        // plasma is dead
        continue;
        } // end if

 ////////////////////////////////////////////////////////


        // test for collision with rocks
        for (rock=0; rock < MAX_ROCKS; rock++)
            {
            if (rocks[rock].state==ROCK_STATE_ON)
                {
                // test for collision 
                if (Collision_Test(plasma[index].varsI[INDEX_WORLD_X]-(plasma[index].width*.5), 
					               plasma[index].varsI[INDEX_WORLD_Y]-(plasma[index].height*.5), 
                                   plasma[index].width, plasma[index].height,
                                   rocks[rock].varsI[INDEX_WORLD_X]-(rocks[rock].width*.5), 
								   rocks[rock].varsI[INDEX_WORLD_Y]-(rocks[rock].height*.5),
                                   rocks[rock].width, rocks[rock].height))
                    {
                    // kill pulse
                    plasma[index].state = PLASMA_STATE_OFF;
                    plasma[index].counter_1 = 0;
  
                   // start a powerup
                   if (RAND_RANGE(0,9) == 5)
                      {
                      // start either type of powerup
                      Start_Powerup(RAND_RANGE(POWERUP_TYPE_AMMO,POWERUP_TYPE_SHLD),
                                     rocks[rock].varsI[INDEX_WORLD_X], 
                                     rocks[rock].varsI[INDEX_WORLD_Y]);
         
                       } // end if

                    switch(rocks[rock].varsI[0])
                          {
                          case ROCK_LARGE:
                              {
                              // start explosion
                              Start_Burst(plasma[index].varsI[INDEX_WORLD_X], plasma[index].varsI[INDEX_WORLD_Y], 
                                          68+rand()%12,54+rand()%10,
                                          rocks[rock].xv*.5, rocks[rock].yv*.5);
                                        
                              } break;

                          case ROCK_MEDIUM:
                              {
                              // start explosion
                              Start_Burst(plasma[index].varsI[INDEX_WORLD_X], plasma[index].varsI[INDEX_WORLD_Y], 
                                          52+rand()%10,44+rand()%8,
                                          rocks[rock].xv*.5, rocks[rock].yv*.5);

                              } break;

                          case ROCK_SMALL:
                              {

                              // start explosion
                              Start_Burst(plasma[index].varsI[INDEX_WORLD_X], plasma[index].varsI[INDEX_WORLD_Y], 
                                          34-4+rand()%8,30-3+rand()%6,
                                          rocks[rock].xv*.5, rocks[rock].yv*.5);

                              } break;
                          
                          } // end switch

                    // update score
                    player_score+=rocks[rock].varsI[2];

                    // test strength of rock, cause damage
                    rocks[rock].varsI[2]-=50;

                    // split test
                    if (rocks[rock].varsI[2] > 0 && rocks[rock].varsI[2] < 50)
                        {
                        // test the size of rock
                        switch(rocks[rock].varsI[0])
                        {
                        case ROCK_LARGE:
                            {
                            // split into two medium
                            Start_Rock(rocks[rock].varsI[INDEX_WORLD_X]+rand()%16,rocks[rock].varsI[INDEX_WORLD_Y]+rand()%16,
                                       ROCK_MEDIUM,
                                       rocks[rock].xv-2+rand()%4,rocks[rock].yv-2+rand()%4);
                    
                            Start_Rock(rocks[rock].varsI[INDEX_WORLD_X]+rand()%16,rocks[rock].varsI[INDEX_WORLD_Y]+rand()%16,
                                       ROCK_MEDIUM,
                                       rocks[rock].xv-2+rand()%4,rocks[rock].yv-2+rand()%4);
                            
                           // throw in a small?
                           if ((rand()%3)==1)
                            Start_Rock(rocks[rock].varsI[INDEX_WORLD_X]+rand()%16,rocks[rock].varsI[INDEX_WORLD_Y]+rand()%16,
                                       ROCK_SMALL,
                                       rocks[rock].xv-2+rand()%4,rocks[rock].yv-2+rand()%4);

                            // kill the original
                            rocks[rock].state = ROCK_STATE_OFF;     
            
                            } break;

                        case ROCK_MEDIUM:
                            {
                            // split into 1 - 3 small
                            int num_rocks = 1+rand()%3;

                            for (; num_rocks >=1; num_rocks--)
                                {
                                Start_Rock(rocks[rock].varsI[INDEX_WORLD_X]+rand()%8,rocks[rock].varsI[INDEX_WORLD_Y]+rand()%8,
                                           ROCK_SMALL,
                                           rocks[rock].xv-2+rand()%4,rocks[rock].yv-2+rand()%4);

                                } // end for num_rocks
                 
                            // kill the original
                            rocks[rock].state = ROCK_STATE_OFF;

                            } break;

                        case ROCK_SMALL:
                            {
                            // just kill it
                            rocks[rock].state = ROCK_STATE_OFF;

                            } break;
 
                        default:break;

        
                        } // end switch

                        } // end if split
                    else
                    if (rocks[rock].varsI[2] <= 0)
                        {
                        // kill rock
                        rocks[rock].state = ROCK_STATE_OFF;
                        } // end else

                    // break out of loop
                    break;

                   } // end if collision

                } // end if rock alive

            } // end for rock

      } // end if

    } // end for index

} // end Move_Plasma

///////////////////////////////////////////////////////////

void Draw_Plasma(void)
{
	int index;
// this function draws all the plasma pulses

for (index=0; index<MAX_PLASMA; index++)
    {
    // test if plasma pulse is in flight
    if (plasma[index].state == PLASMA_STATE_ON)
        {
        // transform to screen coords
        plasma[index].x = plasma[index].varsI[INDEX_WORLD_X] - (plasma[index].width >> 1) - player_x + (SCREEN_WIDTH/2);
        plasma[index].y = plasma[index].varsI[INDEX_WORLD_Y] - (plasma[index].height >> 1) - player_y + (SCREEN_HEIGHT/2);

        // draw the pulse
        NewDraw_BOB(&plasma[index]);
         
        // animate the pulse
        Animate_BOB(&plasma[index]);

        } // end if

    } // end for index

} // end Draw_Plasma

///////////////////////////////////////////////////////////

void Fire_Plasma(int x,int y, int xv, int yv, int source)
{
	int index;
	int sound_index;
// this function fires a plasma pulse at the given starting
// position and velocity, of course, one must be free for 
// this to work

// scan for a pulse that is available
for (index=0; index < MAX_PLASMA; index++)
    {
    // is this one available
      // test if plasma pulse is in flight
    if (plasma[index].state == PLASMA_STATE_OFF)
       {
       // start this one up, note the use of world coords
       plasma[index].varsI[INDEX_WORLD_X] = x-(plasma[0].width*.5);
       plasma[index].varsI[INDEX_WORLD_Y] = y-(plasma[0].height*.5);
       
       plasma[index].xv = xv;
	   plasma[index].yv = yv;
       plasma[index].curr_frame = 0;
       plasma[index].state =  PLASMA_STATE_ON;
       plasma[index].counter_1 = 0; // used to track distance

	   // set animation and typing info for collision engine
	   plasma[index].anim_state = source;
	   Set_Animation_BOB(&plasma[index], source);

       // start sound up
       PlayToneOS5(Pulse);

       // later
       return;

       } // end if

    } // end for

} // end Fire_Plasma


///////////////////////////////////////////////////////////

void Init_Rocks(void)
{
	int rock;
// this function initializes and loads all the rocks 

int frame; // used as loop index

// create the large rock
Create_BOB(&rock_l,0,0,96,96,16,0,BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME | BOB_ATTR_ABS);
Set_Transparent_BOB(&rock_l,true,0,0,0);

// load animation frames
for (frame=0; frame <= 15; frame++)
	{
	// load the rocks imagery 
    //Pad_Name("OUTART/ROCKL", "BMP", buffer, frame);
	//Load_Bitmap_File(&bitmap8bit, buffer);		

    // load the actual .BMP
    Load_Frame_BOB(&rock_l,RockL0 + frame,frame,0,0,BITMAP_EXTRACT_MODE_ABS);  

    // unload data infile
    //Unload_Bitmap_File(&bitmap8bit);

	} // end if

// set animation rate
Set_Anim_Speed_BOB(&rock_l,1+rand()%5);
Set_Vel_BOB(&rock_l, -4+rand()%8, 4+rand()%4);
Set_Pos_BOB(&rock_l, rand()%SCREEN_WIDTH, rand()%SCREEN_HEIGHT);

// set size of rock
rock_l.varsI[0] = ROCK_LARGE;
rock_l.varsI[1] = rock_sizes[rock_l.varsI[0]];

// set state to off
rock_l.state = ROCK_STATE_OFF;

// create the medium rock
Create_BOB(&rock_m,0,0,56,56,16,0,BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME | BOB_ATTR_ABS);
Set_Transparent_BOB(&rock_m,true,0,0,0);

// load animation frames
for (frame=0; frame <= 15; frame++)
	{
	// load the rocks imagery 
    //Pad_Name("OUTART/ROCKM", "BMP", buffer, frame);
	//Load_Bitmap_File(&bitmap8bit, buffer);		

    // load the actual .BMP
    Load_Frame_BOB(&rock_m,RockM0 + frame,frame,0,0,BITMAP_EXTRACT_MODE_ABS);  

    // unload data infile
    //Unload_Bitmap_File(&bitmap8bit);

	} // end if

// set animation rate
Set_Anim_Speed_BOB(&rock_m,1+rand()%5);
Set_Vel_BOB(&rock_m, -4+rand()%8, 4+rand()%4);
Set_Pos_BOB(&rock_m, rand()%SCREEN_WIDTH, rand()%SCREEN_HEIGHT);

// set size of rock
rock_m.varsI[0] = ROCK_MEDIUM;
rock_m.varsI[1] = rock_sizes[rock_m.varsI[0]];

// set state to off
rock_m.state = ROCK_STATE_OFF;

// create the small rock
Create_BOB(&rock_s,0,0,32,32,16,0,BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME | BOB_ATTR_ABS);
Set_Transparent_BOB(&rock_s,true,0,0,0);

// load animation frames
for (frame=0; frame <= 15; frame++)
	{
	// load the rocks imagery 
    //Pad_Name("OUTART/ROCKS", "BMP", buffer, frame);
	//Load_Bitmap_File(&bitmap8bit, buffer);		

    // load the actual .BMP
    Load_Frame_BOB(&rock_s,RockS0 + frame,frame,0,0,BITMAP_EXTRACT_MODE_ABS);  

    // unload data infile
    //Unload_Bitmap_File(&bitmap8bit);

	} // end if

// set animation rate
Set_Anim_Speed_BOB(&rock_s,1+rand()%5);
Set_Vel_BOB(&rock_s, -4+rand()%8, -4+rand()%8);
Set_Pos_BOB(&rock_s, rand()%SCREEN_WIDTH, rand()%SCREEN_HEIGHT);

// set size of rock
rock_s.varsI[0] = ROCK_SMALL;
rock_s.varsI[1] = rock_sizes[rock_s.varsI[0]];

// set state to off
rock_s.state = ROCK_STATE_OFF;

// make semi copies
for (rock=0; rock < MAX_ROCKS; rock++)
    {
    //MemMove(&rocks[rock], &rock_l, sizeof(BOB));
	Clone_BOB(&rock_l,&rocks[rock]);
    // set animation rate
    Set_Anim_Speed_BOB(&rocks[rock],1+rand()%5);
    
    // set velocity
    Set_Vel_BOB(&rocks[rock], -4+rand()%8, -4+rand()%8);
    
    // set position
    Set_Pos_BOB(&rocks[rock],0,0);
    rocks[rock].varsI[INDEX_WORLD_X] = RAND_RANGE(UNIVERSE_MIN_X, UNIVERSE_MAX_X);
    rocks[rock].varsI[INDEX_WORLD_Y] = RAND_RANGE(UNIVERSE_MIN_Y, UNIVERSE_MAX_Y);    

    // set size of rock
    rocks[rock].varsI[0] = rand()%3;
    rocks[rock].varsI[1] = rock_sizes[rocks[rock].varsI[0]];

    // based on size of rock switch in correct bitmaps and sizes
	switch(rocks[rock].varsI[1])
		{
		case ROCK_LARGE:		
			{
            // copy dd bitmap surfaces
            MemMove(rocks[rock].images,rock_l.images,sizeof(TwGfxSurfaceType *)*MAX_BOB_FRAMES);

			// set width and height (incase)
			rocks[rock].width  = rock_l.width;
			rocks[rock].height = rock_l.height;

			} break;

		case ROCK_MEDIUM:
			{
            // copy dd bitmap surfaces
            MemMove(rocks[rock].images,rock_m.images,sizeof(TwGfxSurfaceType *)*MAX_BOB_FRAMES);

			// set width and height (incase)
			rocks[rock].width  = rock_m.width;
			rocks[rock].height = rock_m.height;

			} break;

		case ROCK_SMALL:
			{
            // copy dd bitmap surfaces
            MemMove(rocks[rock].images,rock_s.images,sizeof(TwGfxSurfaceType *)*MAX_BOB_FRAMES);

			// set width and height (incase)
			rocks[rock].width  = rock_s.width;
			rocks[rock].height = rock_s.height;

			} break;
		
		default: break;

		} // end switch
   
    } // end for rock

} // end Init_Rocks

//////////////////////////////////////////////////////////////////////////////

void Reset_Rocks(void)
{
	int rock;
// this function resets all the rocks

for ( rock=0; rock < MAX_ROCKS; rock++)
    {
    // reset state
    rocks[rock].state = ROCK_STATE_OFF;

    } // end for rocks

} // end Reset_Rocks

//////////////////////////////////////////////////////////////////////////////

void Delete_Rocks(void)
{
	int index;
// this function simply deletes all memory and surfaces
// related to the rocks pulses

for ( index=0; index < MAX_ROCKS; index++)
    Destroy_BOB(&rocks[index]);

Destroy_BOB(&rock_l);
Destroy_BOB(&rock_m);
Destroy_BOB(&rock_s);

} // end Delete_Rocks

//////////////////////////////////////////////////////////////////////////////

void Seed_Rocks(void)
{
// this function seeds the universe with rocks

int index; // looping var

   // scan for a rock to initialize
   for (index=0; index < (MAX_ROCKS*.75); index++)
       {
       // is this rock available?
       if (rocks[index].state == ROCK_STATE_OFF)
          {
          // set animation rate
          Set_Anim_Speed_BOB(&rocks[index],1+rand()%5);
    
          // set velocity
          Set_Vel_BOB(&rocks[index], -6+rand()%12, -6+rand()%12);
    
          // set position
          Set_Pos_BOB(&rocks[index],0,0);
		  rocks[index].varsI[INDEX_WORLD_X] = RAND_RANGE(UNIVERSE_MIN_X, UNIVERSE_MAX_X);
		  rocks[index].varsI[INDEX_WORLD_Y] = RAND_RANGE(UNIVERSE_MIN_Y, UNIVERSE_MAX_Y);    
    
          // set size of rock
          rocks[index].varsI[0] = rand()%3;
          rocks[index].varsI[1] = rock_sizes[rocks[index].varsI[0]];
          
          // set strength of rock
          switch(rocks[index].varsI[0])
                {
                case ROCK_LARGE:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 100+rand()%100;

					 // copy dd bitmap surfaces
					 MemMove(rocks[index].images,rock_l.images,sizeof(TwGfxSurfaceType *)*MAX_BOB_FRAMES);


		  			 // set width and height (incase)
		 			 rocks[index].width  = rock_l.width;
					 rocks[index].height = rock_l.height;

                     } break;
 
                case ROCK_MEDIUM:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 40 + rand()%30;

					 // copy dd bitmap surfaces
					 MemMove(rocks[index].images,rock_m.images,sizeof(TwGfxSurfaceType *)*MAX_BOB_FRAMES);


					 // set width and height (incase)
					 rocks[index].width  = rock_m.width;
				     rocks[index].height = rock_m.height;

                     } break;     

                case ROCK_SMALL:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 10;

					 // copy dd bitmap surfaces
		             MemMove(rocks[index].images,rock_s.images,sizeof(TwGfxSurfaceType *)*MAX_BOB_FRAMES);

					 // set width and height (incase)
					 rocks[index].width  = rock_s.width;
					 rocks[index].height = rock_s.height;
                     } break;

                default:break;

                } // end switch

          // turn rock on
          rocks[index].state = ROCK_STATE_ON;

          } // end if

       } // end for index

} // end Seed_Rocks


//////////////////////////////////////////////////////////

void Move_Rocks(void)
{
	int index;
// this function moves all the rocks pulses and checks for
// collision with the rocks

for (index=0; index < MAX_ROCKS; index++)
    {
    // test if rocks pulse is in flight
    if (rocks[index].state == ROCK_STATE_ON)
        {
        // move the rock
        rocks[index].varsI[INDEX_WORLD_X]+=rocks[index].xv;
        rocks[index].varsI[INDEX_WORLD_Y]+=rocks[index].yv;

        // test if rock is out of universe, but persist object
        if (rocks[index].varsI[INDEX_WORLD_X] > UNIVERSE_MAX_X)
            rocks[index].varsI[INDEX_WORLD_X] = UNIVERSE_MIN_X;
        else
        if (rocks[index].varsI[INDEX_WORLD_X] < UNIVERSE_MIN_X)
            rocks[index].varsI[INDEX_WORLD_X] = UNIVERSE_MAX_X;

        if (rocks[index].varsI[INDEX_WORLD_Y] > UNIVERSE_MAX_X)
            rocks[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MIN_X;
        else
        if (rocks[index].varsI[INDEX_WORLD_Y] < UNIVERSE_MIN_Y)
            rocks[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MAX_Y;
    

        // test for collision with rocks
        if (player_state == PLAYER_STATE_ALIVE && Collision_Test(player_x-(wraith.width*.5), player_y-(wraith.height*.5), 
                           wraith.width, wraith.height,
                           rocks[index].varsI[INDEX_WORLD_X]-(rocks[index].width*.5), 
						   rocks[index].varsI[INDEX_WORLD_Y]-(rocks[index].height*.5),
                           rocks[index].width, rocks[index].height))
        {
        // what size rock did we hit?
        switch(rocks[index].varsI[0])
              {
              case ROCK_LARGE:
                   {
                   // start explosion
                   Start_Burst(rocks[index].varsI[INDEX_WORLD_X], rocks[index].varsI[INDEX_WORLD_Y], 
                               68+rand()%12,54+rand()%10,
                               rocks[index].xv*.5, rocks[index].yv*.5);
                   
                    // update players damage
                    player_damage+=35;

					// update velocity vector
                    player_xv+=(rocks[index].xv);
					player_yv+=(rocks[index].yv);

                    // rotate ship a bit
					wraith.varsI[WRAITH_INDEX_DIR]+=(RAND_RANGE(-4,4));
                    if (wraith.varsI[WRAITH_INDEX_DIR] > 15)
						wraith.varsI[WRAITH_INDEX_DIR] -= 15;
					else
                    if (wraith.varsI[WRAITH_INDEX_DIR] < 0)
						wraith.varsI[WRAITH_INDEX_DIR] += 15;


                    } break;

               case ROCK_MEDIUM:
                    {
                    // start explosion
                    Start_Burst(rocks[index].varsI[INDEX_WORLD_X], rocks[index].varsI[INDEX_WORLD_Y], 
                                52+rand()%10,44+rand()%8,
                                rocks[index].xv*.5, rocks[index].yv*.5);

                    // update players damage
                    player_damage+=15;

					// update velocity vector
                    player_xv+=(rocks[index].xv*.5);
					player_yv+=(rocks[index].yv*.5);

                    // rotate ship a bit
					wraith.varsI[WRAITH_INDEX_DIR]+=(RAND_RANGE(-4,4));
                    if (wraith.varsI[WRAITH_INDEX_DIR] > 15)
						wraith.varsI[WRAITH_INDEX_DIR] -= 15;
					else
                    if (wraith.varsI[WRAITH_INDEX_DIR] < 0)
						wraith.varsI[WRAITH_INDEX_DIR] += 15;

                    } break;

                case ROCK_SMALL:
                     {
                     // start explosion
                     Start_Burst(rocks[index].varsI[INDEX_WORLD_X], rocks[index].varsI[INDEX_WORLD_Y], 
                                 34-4+rand()%8,30-3+rand()%6,
                                 rocks[index].xv*.5, rocks[index].yv*.5);

                      // update players damage
                      player_damage+=5;

  					  // update velocity vector
                      player_xv+=(rocks[index].xv*.25);
 					  player_yv+=(rocks[index].yv*.25);

                    // rotate ship a bit
					wraith.varsI[WRAITH_INDEX_DIR]+=(RAND_RANGE(-4,4));
                    if (wraith.varsI[WRAITH_INDEX_DIR] > 15)
						wraith.varsI[WRAITH_INDEX_DIR] -= 15;
					else
                    if (wraith.varsI[WRAITH_INDEX_DIR] < 0)
						wraith.varsI[WRAITH_INDEX_DIR] += 15;


                      } break;
                          
                } // end switch

                // update score
                player_score+=rocks[index].varsI[2];

                // engage shields
                player_shield_count=3;

                // kill the original
                rocks[index].state = ROCK_STATE_OFF;     

            } // end if collision

		} // end if rock

    } // end for index

// now test if it's time to add a new rock to the list
if ((rand()%100)==50)
   {
   // scan for a rock to initialize
   for (index=0; index < MAX_ROCKS; index++)
       {
       // is this rock available?
       if (rocks[index].state == ROCK_STATE_OFF)
          {
          // set animation rate
          Set_Anim_Speed_BOB(&rocks[index],1+rand()%5);
    
          // set velocity
          Set_Vel_BOB(&rocks[index], -6+rand()%12, -6+rand()%12);
    
          // set position
          Set_Pos_BOB(&rocks[index],0,0);
		  rocks[index].varsI[INDEX_WORLD_X] = RAND_RANGE(UNIVERSE_MIN_X, UNIVERSE_MAX_X);
		  rocks[index].varsI[INDEX_WORLD_Y] = RAND_RANGE(UNIVERSE_MIN_Y, UNIVERSE_MAX_Y);    
    
          // set size of rock
          rocks[index].varsI[0] = rand()%3;
          rocks[index].varsI[1] = rock_sizes[rocks[index].varsI[0]];
          
          // set strength of rock
          switch(rocks[index].varsI[0])
                {
                case ROCK_LARGE:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 100+rand()%100;

					 // copy dd bitmap surfaces
					 MemMove(rocks[index].images,rock_l.images,sizeof(TwGfxSurfaceType *)*MAX_BOB_FRAMES);


		  			 // set width and height (incase)
		 			 rocks[index].width  = rock_l.width;
					 rocks[index].height = rock_l.height;

                     } break;
 
                case ROCK_MEDIUM:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 40 + rand()%30;

					 // copy dd bitmap surfaces
					 MemMove(rocks[index].images,rock_m.images,sizeof(TwGfxSurfaceType *)*MAX_BOB_FRAMES);


					 // set width and height (incase)
					 rocks[index].width  = rock_m.width;
				     rocks[index].height = rock_m.height;

                     } break;     

                case ROCK_SMALL:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 10;

					 // copy dd bitmap surfaces
		             MemMove(rocks[index].images,rock_s.images,sizeof(TwGfxSurfaceType *)*MAX_BOB_FRAMES);

					 // set width and height (incase)
					 rocks[index].width  = rock_s.width;
					 rocks[index].height = rock_s.height;
                     } break;

                default:break;

                } // end switch


          // turn rock on
          rocks[index].state = ROCK_STATE_ON;
          
          // later
          return;

          } // end if

       } // end for index

   } // end if

} // end Move_Rocks

///////////////////////////////////////////////////////////

void Start_Rock(int x, int y, int size,int xv, int yv)
{
	int index;
// this functions starts a rock up with the sent parms, considering
// there is one left

// scan for a rock to initialize
   for (index=0; index < MAX_ROCKS; index++)
       {
       // is this rock available?
       if (rocks[index].state == ROCK_STATE_OFF)
          {
          // set animation rate
          Set_Anim_Speed_BOB(&rocks[index],1+rand()%5);
    
          // set velocity
          Set_Vel_BOB(&rocks[index], xv,yv);
    
          // set position
          Set_Pos_BOB(&rocks[index], 0,0);
          rocks[index].varsI[INDEX_WORLD_X] = x;
          rocks[index].varsI[INDEX_WORLD_Y] = y;
    
          // set size of rock
          rocks[index].varsI[0] = size;
          rocks[index].varsI[1] = rock_sizes[rocks[index].varsI[0]];
          
          // set strength of rock
          switch(rocks[index].varsI[0])
                {
                case ROCK_LARGE:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 100+rand()%100;

			 	     // copy dd bitmap surfaces
		             MemMove(rocks[index].images,rock_l.images,sizeof(TwGfxSurfaceType *)*MAX_BOB_FRAMES);

					 // set width and height (incase)
					 rocks[index].width  = rock_l.width;
					 rocks[index].height = rock_l.height;

                     } break;
 
                case ROCK_MEDIUM:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 40 + rand()%30;

					 // copy dd bitmap surfaces
					 MemMove(rocks[index].images,rock_m.images,sizeof(TwGfxSurfaceType *)*MAX_BOB_FRAMES);

					 // set width and height (incase)
					 rocks[index].width  = rock_m.width;
					 rocks[index].height = rock_m.height;
                     } break;     

                case ROCK_SMALL:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 10;

			         // copy dd bitmap surfaces
					 MemMove(rocks[index].images,rock_s.images,sizeof(TwGfxSurfaceType *)*MAX_BOB_FRAMES);

					 // set width and height (incase)
					 rocks[index].width  = rock_s.width;
					 rocks[index].height = rock_s.height;
                     } break;

                default:break;

                } // end switch

          // turn rock on
          rocks[index].state = ROCK_STATE_ON;
          
          // later
          return;

          } // end if

       } // end for index

} // end Start_Rock

///////////////////////////////////////////////////////////

void Draw_Rocks(void)
{
	int index;
// this function draws all the rocks 

for (index=0; index<MAX_ROCKS; index++)
    {
    // test if rocks pulse is in flight
    if (rocks[index].state == ROCK_STATE_ON)
        {
        // transform to screen coords
        rocks[index].x = rocks[index].varsI[INDEX_WORLD_X] - (rocks[index].width*.5) - player_x + (SCREEN_WIDTH/2);
        rocks[index].y = rocks[index].varsI[INDEX_WORLD_Y] - (rocks[index].height*.5) - player_y + (SCREEN_HEIGHT/2);

        // draw the rock
        NewDraw_BOB(&rocks[index]);
         
        // animate the pulse
        Animate_BOB(&rocks[index]);

        } // end if

    } // end for index

} // end Draw_Rocks

///////////////////////////////////////////////////////////

void Init_Bursts(void)
{
	int burst;
// this function initializes and loads all the bursts 

// load the bursts imagery 
//Load_Bitmap_File(&bitmap8bit, "OUTART/EXPL8.BMP");

// create the first bob
Create_BOB(&bursts[0],0,0,42,36,14,6,BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME | BOB_ATTR_CELL);
Set_Transparent_BOB(&bursts,true,0,0,0);

// load animation frames
//for (int frame=0; frame < 14; frame++)
     Load_Frame_BOB(&bursts[0],EXPL8,0,0,0,BITMAP_EXTRACT_MODE_ABS);  

// set animation rate
Set_Anim_Speed_BOB(&bursts[0],1);

// set size of burst
bursts[0].varsI[0] = bursts[0].width;
bursts[0].varsI[1] = bursts[0].height;

// set state to off
bursts[0].state = BURST_STATE_OFF;

for (burst=1; burst<MAX_BURSTS; burst++)
    {
		//MemMove(&bursts[burst], &bursts[0], sizeof(BOB));
		Clone_BOB(&bursts[0],&bursts[burst]);
    } // end for burst

// unload data infile
//Unload_Bitmap_File(&bitmap8bit);

} // end Init_Bursts

///////////////////////////////////////////////////////////

void Reset_Bursts(void)
{
	int burst;
// this function resets all the bursts

for (burst=0; burst < MAX_BURSTS; burst++)
    {
    // set state to off
    bursts[burst].state = BURST_STATE_OFF;
    } // end for burst


} // end Reset_Bursts


////////////////////////////////////////////////////////////

void Delete_Bursts(void)
{
	int index;
// this function simply deletes all memory and surfaces
// related to the bursts pulses

for (index=0; index < MAX_BURSTS; index++)
    Destroy_BOB(&bursts[index]);

} // end Delete_Bursts

///////////////////////////////////////////////////////////

void Move_Bursts(void)
{
	int index;
// this function moves all the bursts 

for (index=0; index < MAX_BURSTS; index++)
    {
    // test if bursts pulse is in moving
    if (bursts[index].state == BURST_STATE_ON)
       {
       // move the burst
       // Move_BOB(&bursts[index]);
       bursts[index].varsI[INDEX_WORLD_X]+=bursts[index].xv;
       bursts[index].varsI[INDEX_WORLD_Y]+=bursts[index].yv;

        // test if burst is off screen or done with animation
        if (bursts[index].curr_frame >= bursts[index].num_frames-1) 
           {
           // kill burst and put back on available list
           bursts[index].state = BURST_STATE_OFF;
           } // end if
        
        } // end if

    } // end for index

} // end Move_Bursts

///////////////////////////////////////////////////////////

void Draw_Bursts(void)
{
	int index;
// this function draws all the bursts 
for (index=0; index < MAX_BURSTS; index++)
    {
    // test if bursts pulse is in flight
    if (bursts[index].state == BURST_STATE_ON)
        {
		// transform bursts to screen space
        bursts[index].x = bursts[index].varsI[INDEX_WORLD_X] - (bursts[index].width*.5) - player_x + (SCREEN_WIDTH/2);
        bursts[index].y = bursts[index].varsI[INDEX_WORLD_Y] - (bursts[index].height*.5) - player_y + (SCREEN_HEIGHT/2);

		// is scaling needed
        if (bursts[index].varsI[0]!=bursts[index].width || 
            bursts[index].varsI[1]!=bursts[index].height)
            {
            // draw the burst scaled
            Draw_Scaled_BOB(&bursts[index],bursts[index].varsI[0],bursts[index].varsI[1]);
            }
        else // draw normal
           NewDraw_BOB(&bursts[index]);
         
        // animate the explosion
        Animate_BOB(&bursts[index]);

        } // end if

    } // end for index

} // end Draw_Bursts

///////////////////////////////////////////////////////////

void Start_Burst(int x, int y, int width, int height, int xv,int yv)
{
	int index;
	int sound_index;
// this function starts a burst up

// now test if it's time to add a new burst to the list

// scan for a burst to initialize
for (index=0; index < MAX_BURSTS; index++)
    {
    // is this burst available?
    if (bursts[index].state == BURST_STATE_OFF)
       {
       // set animation rate
       Set_Anim_Speed_BOB(&bursts[index],1);
       bursts[index].curr_frame = 0;
    
       // set velocity
       Set_Vel_BOB(&bursts[index], xv,yv);
    
       // set position
       Set_Pos_BOB(&bursts[index], 0,0);
	   bursts[index].varsI[INDEX_WORLD_X] = x;
	   bursts[index].varsI[INDEX_WORLD_Y] = y;
    
       // set size of burst
       bursts[index].varsI[0] = width;
       bursts[index].varsI[1] = height;

       // turn burst on
       bursts[index].state = BURST_STATE_ON;

       // shoot some particles out
       Start_Particle_Explosion(PARTICLE_TYPE_FLICKER, range2(PARTICLE_COLOR_RED,PARTICLE_COLOR_GREEN),20+rand()%40, 
                                bursts[index].varsI[INDEX_WORLD_X], bursts[index].varsI[INDEX_WORLD_Y], 
                                bursts[index].xv, bursts[index].yv, 
                                8+rand()%8); 


       // start sound up
       PlayToneOS5(range2(Expl1,Expl2));
       // later
       return;

       } // end if
 
    } // end for index

} // end Start_Burst

///////////////////////////////////////////////////////////

void Draw_Info(void)
{
// this function draws all the information at the top of the screen

char score[16]; // hold score
static int red_glow = 0; // used for damage display
static int warning_count = 0;
char buffer[80];

// build up scrore string
sprintf(score,"0000000%d",player_score);

// build up final string
sprintf(buffer,"SCORE %s",&score[strlen(score)-8]);
Draw_Text(buffer,StrLen(buffer),0,294);

// draw damage
sprintf(buffer,"SHIELDS %d%%",100-player_damage);

if (player_damage < 90)
   Draw_Text(buffer,strlen(buffer),0,0);
else
  Draw_Text(buffer,strlen(buffer),0,0);

// update red glow
if ((red_glow+=15) > 255)
   red_glow = 0;

// draw ship info
sprintf(buffer,"SHIPS %d",player_ships);
Draw_Text(buffer,strlen(buffer),400,0);

/*sprintf(buffer,"VEL %.2f Kps",vel*(100/MAX_PLAYER_SPEED));
Draw_Text_GDI(buffer,10,460,RGB(0,255,0),lpddsback);

sprintf(buffer,"POS [%.2f,%.2f]",player_x, player_y);
Draw_Text_GDI(buffer,480,460,RGB(0,255,0),lpddsback);
*/
sprintf(buffer,"AMMO %d",player_ammo);
Draw_Text(buffer,strlen(buffer),340,294);

} // end Draw_Info

///////////////////////////////////////////////////////////////////

void Do_Intro(void)
{
// the worlds simples intro
/*
// clear out buffers
DDraw_Fill_Surface(lpddsback, 0);
DDraw_Fill_Surface(lpddsprimary, 0);

// draw in logo screen
Load_Bitmap_File(&bitmap8bit, "OUTART/OUTPOSTSPLASH.BMP");

// set the palette to background image palette
Set_Palette(bitmap8bit.palette);

// copy the bitmap to primary buffer
DDraw_Lock_Primary_Surface();
Copy_Screen(bitmap8bit.buffer,primary_buffer, primary_lpitch, 0);
DDraw_Unlock_Primary_Surface();

// unload bitmap file
Unload_Bitmap_File(&bitmap8bit);

Sleep(5000);

// transition to black
Screen_Transitions(SCREEN_DARKNESS,NULL,0);

// clear out buffers
DDraw_Fill_Surface(lpddsback, 0);
DDraw_Fill_Surface(lpddsprimary, 0);
*/
} // end Do_Intro

void Load_HUD(void)
{
// this function loads the animation for the hud

int index; // looping variable
	
// load the wraith ship
//Load_Bitmap_File(&bitmap8bit, "OUTART/HUDART8.BMP");

// now create the wraith
Create_BOB(&hud,0,0,24,26,4,24,BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME | BOB_ATTR_CELL);
Set_Transparent_BOB(&hud,true,0,0,0);

// load hud frames
//for (index=0; index < 4; index++)
     Load_Frame_BOB(&hud,HUDART8,0,0,0,BITMAP_EXTRACT_MODE_ABS);  

// unload data infile
//Unload_Bitmap_File(&bitmap8bit);

} // end Load_HUD

///////////////////////////////////////////////////////////////////////////////

int Load_Player(void)
{
// this function loads the animation for the player

int index; // looping variable
	
// load the wraith ship
//Load_Bitmap_File(&bitmap8bit, "OUTART/WRAITHB8.BMP");

// set the palette to background image palette
//Set_Palette(bitmap8bit.palette);

// save this palette
//Save_Palette(game_palette);

// now create the wraith
Create_BOB(&wraith,0,0,64,64,35,8,BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME | BOB_ATTR_CELL);
Set_Transparent_BOB(&wraith,true,0,0,0);
// load wraith frames
//for (index=0; index < 35; index++)
     Load_Frame_BOB(&wraith,WRAITHB8,0,0,0,BITMAP_EXTRACT_MODE_ABS);  

// set position
Set_Pos_BOB(&wraith,(SCREEN_WIDTH/2)-wraith.width/2,(SCREEN_HEIGHT/2)-wraith.height/2);

// set starting direction
wraith.varsI[WRAITH_INDEX_DIR] = 0;

// unload data infile
//Unload_Bitmap_File(&bitmap8bit);

// return success
return(1);

} // end Load_Player

////////////////////////////////////////////////////////////////////

void Create_Tables(void)
{
// this function creates all the lookup tables for the game
float fang;
int ang = 0; // looping var to track angle

// create the 16 sector sin, cos lookup
for (ang=0; ang < 16; ang++)
	{
	fang = PI*(ang*22.5)/180;

	cos_look16[ang] = -cos(fang+PI/2);
	sin_look16[ang] = -sin(fang+PI/2);

	} // end for ang

} // end Create_Tables

/////////////////////////////////////////////////////////////////////////////////////

float Fast_Distance_2D(float x, float y)
{
	float mn;
// this function computes the distance from 0,0 to x,y with 3.5% error

// first compute the absolute value of x,y
x = fabs(x);
y = fabs(y);

// compute the minimum of x,y
mn = MIN(x,y);

// return the distance
return(x+y-(mn/2)-(mn/4)+(mn/8));

} // end Fast_Distance_2D

///////////////////////////////////////////////////////////

void Game_Reset(void)
{
// this function resets all the game related systems and vars for another run

int index; // looping var

// player state variables
player_state       = PLAYER_STATE_ALIVE;
player_score       = 0;   // the score
player_ships       = 3;   // ships left
player_regen_counter = 0; // used to track when to regen
player_damage      = 0;   // damage of player
player_counter     = 0;   // used for state transition tracking
player_ammo        = 1000; // ammo
player_regen_count = 0;   // used to regenerate player
player_shield_count = 0;  // used to display shields
win_counter = 0;
player_won = 0;   
num_stations_destroyed = 0; // number of stations destroyed thus far

// menu_up = 0;             // flags if the main menu is up
// intro_done = 0;          // flags if intro has played already

// positional and state info for player
player_x=0;
player_y=0;
player_dx=0;
player_dy=0; 
player_xv=0; 
player_yv=0;
vel = 0;
wraith.varsI[WRAITH_INDEX_DIR] = 0;
game_state = GAME_STATE_RESTART;   // initial game state
station_id  = -1;

ready_counter = 0,        // used to draw a little "get ready"
ready_state   = 0;

// call all the reset funcs
Init_Reset_Particles();
Reset_Gunships();
Reset_Mines();
Reset_Rocks();
Reset_Plasma();
Reset_Bursts();
Reset_Stations();
Reset_Powerups();

// start the rocks
Seed_Rocks();


// start the gunships
for (index=0; index < NUM_ACTIVE_GUNSHIPS; index++)
    Start_Gunship(0,0,0,0);

// start the mines
for (index=0; index < NUM_ACTIVE_MINES; index++)
    Start_Mine(0,0,0,MINE_STATE_AI_ACTIVATED);

// start the stations
for (index=0; index < NUM_ACTIVE_STATIONS; index++)
    Start_Station(0,0,0);
 
} // end Game_Reset

////////////////////////////////////////////////////////////

int Game_Init()
{
// this function is where you do all the initialization 
// for your game

int index;         // looping var
int ang = 0;
//char filename[80]; // used to build up files names

// generate the tables
for (ang = 0; ang < 360; ang++)
	{
	// convert ang to radians
	float theta = (float)ang*PI/(float)180;

	// insert next entry into table
	cos_look[ang] = cos(theta);
	sin_look[ang] = sin(theta);

	} // end for ang


//mono.clear();
//mono.print("\ndebugger on-line\n");

// seed random number generate
//srand(Start_Clock());

// create all lookup tables
Create_Tables();

// initialize directdraw
//DDraw_Init(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP);

// initialize directsound
//DSound_Init();

// initialize Directinput
//DInput_Init();

// initialize all input devices
//DInput_Init_Keyboard();
 
// load all the sound and music
//Load_Sound_Music();

// initialize the plasma pulses
Init_Plasma();

// initialize all the stars
Init_Stars();

// init rocks
Init_Rocks();
Seed_Rocks();

// init all the explosions
Init_Bursts();

// initialize all the Mines
Init_Mines();

// initialize all the gunships
Init_Gunships();

// initialize all the stations, must be after mines!
Init_Stations();

// initialize the powerups
Init_Powerups();

// intialize particle system
Init_Reset_Particles();

//What is this?  AK
//Load_Andre();

// set clipping rectangle to screen extents so objects dont
// mess up at edges
//RECT screen_rect = {0,0,screen_width,screen_height};
//lpddclipper = DDraw_Attach_Clipper(lpddsback,1,&screen_rect);

// hide the mouse
//ShowCursor(FALSE);

// start the intro music
startMusic(true);

// do the introdcution
Do_Intro();

// load the player
Load_Player();

// load the hud art
Load_HUD();

// return success
return(1);

} // end Game_Init

///////////////////////////////////////////////////////////

int Game_Shutdown()
{
// this function is where you shutdown your game and
// release all resources that you allocated

// delete all the explosions
Delete_Bursts();
    
// delete the player
Destroy_BOB(&wraith);

// delete all the rocks
Delete_Rocks();

// delete all the plasma pulses
Delete_Plasma();

// delete all the stations
Delete_Stations();

// delete the mines
Delete_Mines();

// shutdown directdraw
//DDraw_Shutdown();

// unload sounds
//DSound_Delete_All_Sounds();

stopMusic();

// shutdown directsound
//DSound_Shutdown();

// release all input devices
//DInput_Release_Keyboard();

// return success
return(1);
} // end Game_Shutdown

///////////////////////////////////////////////////////////


//-----------------------------------------------------------------------------------------

void
HostMaskKeys()
{
    if (!GameApp.isInputMasked)
    {
        // Set the keys we poll to not generate events.  This saves cpu cycles
        // by avoiding the events flowing through the event loop which we will 
        // ignore anyways.
        KeySetMask(~keysAllowedMask);
        
        
        if (TapWaveSupport)
        {
            // Open an input queue for the joystick.  The queue is polled before
            // advancing each game cycle.
            TwInputOpen(&GameApp.inputQueueP, "twinput", "r");
            
            // The input format must be four byte aligned.  A 
            // simple way to do this for all compilers is to allocate it and
            // move the data into it.  Metrowerks compilers (>= version 9)
            // can use "#pragma options align=4".  Gcc compilers can use 
            // "static Int32 InputFormats[] __attribute__ ((aligned (4))) = {...};"
            GameApp.inputFormatP = MemPtrNew(sizeof(InputFormats));
            MemMove(GameApp.inputFormatP, InputFormats, sizeof(InputFormats));
            TwInputSetFormat(GameApp.inputQueueP, GameApp.inputFormatP, sizeof(InputFormats));
            
            // Activating the queue stops the joystick from generating
            // keyDownEvents.  If this is not done, then every direction
            // visited while polling will cause a corresponding keyDownEvent
            // in that direction, and if the polling duration exceeds 
            // the keyDown repeat duration, then a repeating keyDownEvent 
            // will be generated too.
            TwInputActivate(GameApp.inputQueueP);
        }
        
        
        GameApp.isInputMasked = true;
    }
    
    return;
}


/*! \brief  Unmask the keys so they can generate events again.

Also, closes the joystick so it can generate events again.

\see HostMaskKeys
*/
void
HostUnmaskKeys()
{
    if (GameApp.isInputMasked)
    {
        // Set the keys we poll to not generate events.  This saves cpu cycles.
        KeySetMask(keyBitsAll);
        
        if (TapWaveSupport)
        {
            TwInputClose(GameApp.inputQueueP);
            
            MemPtrFree(GameApp.inputFormatP);
        }
        
        GameApp.isInputMasked = false;
    }
}


/*! \brief	Send a data stream to the rumbler.

Used at the same time that the buffer is transferred to the screen, to keep 
the tactile stream in sync with the visual stream.
*/
typedef struct
{
	void * data;	// Volume & duration pairs.
	Int32 size;
} EffectInfo;

void
HostPlayRumble(short int effect)
{
    // A real scheme would important data from resources.
    
    // A single, short bump.
    UInt8 EffectBump[] = 
    {
    	128 + 64, 6, 0, 50
	};
    EffectInfo EffectData[1];
    EffectData[0].data = EffectBump;
    EffectData[0].size = sizeof(EffectBump);

    // Send to the rumbler 
    TwDeviceControl(GameApp.rumblerH, 'play', 
    	EffectData[effect].data, 
    	EffectData[effect].size);
}


/*! \brief Create an offscreen buffer of the requested size.

Any existing buffer is freed.  This buffer is also freed by AppStop
before the app exits.

\see HostOpenScreenBuffer
\see HostDrawObject
\see HostCloseScreenBuffer

*/
void
HostNewScreenBuffer(short int buffer_width,
    short int buffer_height)
{
    TwGfxSurfaceInfoType surfInfo;
    Err error;


    if (GameApp.screenBufferH)
    {
        TwGfxFreeSurface(GameApp.screenBufferH);
        GameApp.screenBufferH = NULL;
    }

    // Setup the screenBuffer.  Clear it for use.
    surfInfo.size = sizeof(surfInfo);
    surfInfo.width = buffer_width;
    surfInfo.height = buffer_height;
    surfInfo.location = twGfxLocationAcceleratorMemory;
    surfInfo.pixelFormat = twGfxPixelFormatRGB565;
    error = TwGfxAllocSurface(GameApp.GFX, &GameApp.screenBufferH, &surfInfo);

    GameApp.screenBufferP = NULL;
}

/*! \brief	Set draws to occur at the screen buffer

\see HostNewScreenBuffer
\see HostDrawObject
\see HostDrawGameMenu
\see HostDrawBuffer
\see HostCloseScreenBuffer
*/
void
HostOpenScreenBuffer(void)
{
    ErrNonFatalDisplayIf(GameApp.screenBufferH == NULL ||
        GameApp.screenBufferP != NULL, "Open screen");

    // Set the draw window to the screenBuffer so that all the draw operations act on the
    // screenBuffer which we draw later.
    if (GameApp.customBlitting)
        TwGfxLockSurface(GameApp.screenBufferH, (void**)&GameApp.screenBufferP);
}


void
FPSAdd(FPSType * fps, Int32 time)
{
	Int32 tooOld;
	
	
	if (fps->count == 0)
		fps->time_of_last_data_out = time;
	
	else if (fps->count == FPS_DATA_MAX)
		fps->count--;
	
	// discard data that's too old;
#if CPU_TYPE == CPU_x86
	tooOld = time - 10000;
#else
	tooOld = time - 1000;
#endif
	while (fps->count >= 1 && fps->time[fps->count - 1] < tooOld)
		fps->count--;
		
	MemMove(&fps->time[1], &fps->time[0], 
		sizeof(*fps->time) * fps->count);
	
    fps->count++;
	fps->time[0] = time;
}


void
DrawGauges(short int destX,
    short int destY)
{
    Int32 time;
	
	
	// Draw Gauges.  This adds a variable amount of time to the start 
	// of the game cycle.  This should probably be moved to the end of the cycle.
	GaugeDraw(GameApp.SPFGauge, GameApp.screenBufferH, 
		TwGfxComponentsToPackedRGB(0xff, 0, 0),
		destX, 315 + destY);
	
	time = TimGetTicks();
	FPSAdd(&GameApp.fps, time);
	if (time - GameApp.fps.time_of_last_data_out >= 1000)
	{
		GameApp.fps.time_of_last_data_out = time;
        GaugeAdd(GameApp.FPSGauge, GameApp.fps.count);

	}
	GaugeDraw(GameApp.FPSGauge, GameApp.screenBufferH, 
		TwGfxComponentsToPackedRGB(0, 0xff, 0),
		destX, 285 + destY);
}


/* \brief	Draw the offscreen buffer to the screen.

Used after all drawing is done and typically after the buffer is closed to
transfer the results to the screen.
*/
#define screenTopLeftXOffset		0
#define screenTopLeftYOffset		0
void
HostDrawBuffer(short int destX,
    short int destY,
    short int sourceX,
    short int sourceY,
    short int sourceWidth,
    short int sourceHeight)
{
    TwGfxRectType sourceBounds;
    TwGfxPointType destPoint;
	
	//DrawGauges(sourceX, sourceY);
    
    sourceBounds.x = sourceX;
    sourceBounds.y = sourceY;
    sourceBounds.w = sourceWidth;
    sourceBounds.h = sourceHeight;
    destPoint.x = screenTopLeftXOffset + destX;
    destPoint.y = screenTopLeftYOffset + destY;

    // Copy the screenBuffer.  Have it wait until the 
    // next VSync to copy cleanly visually, and to give
    // improve performance.
    TwGfxBitBlt(GameApp.displaySurface, &destPoint,
                GameApp.screenBufferH, &sourceBounds);
}

/*! \brief Get the width of the Palm bitmap.

The result is in the native coordinate system (480x320).
NOTE: It's probably worthwhile to simply cache this information
after allocating the bitmap.

\see HostDrawObject
*/

// Assertion Support
static void
assert_check(int error, const char* message)
{
    if (error) {
        ErrDisplayFileLineMsg("PalmOS", (UInt16) error, message);
    }
}

#define xassert(error) assert_check(error, #error);

/*static void showError(const char* aWhat, Err aError) 
{
    ErrDisplayFileLineMsg("oink", (UInt16) aError, aWhat);
}

static void showError(const char* aWhat, UInt32 aValue) 
{
    char chbuf[50];
    StrPrintF(chbuf, "%lx", aValue);
    ErrDisplayFileLineMsg(aWhat, (UInt16)0, chbuf);
}*/


TwGfxSurfaceType* LoadBitmap(Int16 aResID)
{
    TwGfxSurfaceType* backsurf = NULL;
    TwGfxSurfaceInfoType surfInfo;
	TwGfxPointType dp= {0, 0};

#ifdef __PALMOS_ARMLET__
    MemHandle resH = DmGetResource('Tbmp', aResID);
#else
    MemHandle resH = DmGetResource('abmp', aResID);
#endif
    if (resH != NULL) {
        BitmapType* resBmp = (BitmapType*) MemHandleLock(resH);
        if (resBmp) {
            UInt16 prevcs = WinSetCoordinateSystem(kCoordinatesNative);

            // Get information about bitmap
            Coord w, h;
            WinGetBitmapDimensions(resBmp, &w, &h);

            // Allocate a surface to hold the image
            surfInfo.size = sizeof(surfInfo);
            surfInfo.width = w;
            surfInfo.height = h;
            surfInfo.location = twGfxLocationAcceleratorMemory;
            surfInfo.pixelFormat = twGfxPixelFormatRGB565;
            xassert(TwGfxAllocSurface(GameApp.GFX, &backsurf, &surfInfo));

            xassert(TwGfxDrawPalmBitmap(backsurf, &dp, resBmp));
            xassert(MemHandleUnlock(resH));

            WinSetCoordinateSystem(prevcs);
        } else {
            assert_check(aResID, "MemHandleUnlock failed");
        }
        xassert(DmReleaseResource(resH));
    } else {
        assert_check(aResID, "DmGetResource failed");
    }

    return backsurf;
}

int
HostGetBitmapWidth(int bitmapNumber)
{
	TwGfxSurfaceInfoType surfInfo;
	
    surfInfo.size = sizeof(surfInfo);
    TwGfxGetSurfaceInfo(GameApp.pictureBitmapP[bitmapNumber], &surfInfo);
    
    return surfInfo.width;
}


/*! \brief Get the height of the Palm bitmap.

The result is in the native coordinate system (480x320).
NOTE: It's probably worthwhile to simply cache this information
after allocating the bitmap.

\see HostDrawObject
*/

int
HostGetBitmapHeight(int bitmapNumber)
{
	TwGfxSurfaceInfoType surfInfo;
	
    surfInfo.size = sizeof(surfInfo);
    TwGfxGetSurfaceInfo(GameApp.pictureBitmapP[bitmapNumber], &surfInfo);
    
    return surfInfo.height;
}


/*! \brief Draw the picture directly to the offscreen buffer.

This is a private function called when customBlitting is true.
It is intended as an example of how to get at the offscreen buffer's bits.

\see HostDrawObject
*/
static void
FastDrawPicture(int picture,
   int x,
   int y)
{
   UInt8 p;
   Int32 screenWidth = GameApp.screenWidth;


   if (GameApp.pictureIsTransparent[picture])
   {
      UInt8 *src;
      UInt8 *dest;
      UInt8 index = (UInt8) GameApp.pictureTransparencyIndex[picture];
      int row;

      src = GameApp.pictureBitmapBitsP[picture];
      dest = &GameApp.screenBufferP[y * GameApp.screenHeight + x];

      for (row = 15; row > 0; row--)
      {
         p = *src++;
         if (p != index)
            *dest = p;
         dest++;

         p = *src++;
         if (p != index)
            *dest = p;
         dest++;

         p = *src++;
         if (p != index)
            *dest = p;
         dest++;

         p = *src++;
         if (p != index)
            *dest = p;
         dest++;

         p = *src++;
         if (p != index)
            *dest = p;
         dest++;

         p = *src++;
         if (p != index)
            *dest = p;
         dest++;

         p = *src++;
         if (p != index)
            *dest = p;
         dest++;

         p = *src++;
         if (p != index)
            *dest = p;
         dest++;

         p = *src++;
         if (p != index)
            *dest = p;
         dest++;

         p = *src++;
         if (p != index)
            *dest = p;
         dest++;

         (UInt8 *) dest += screenWidth;
      }
   }
   else
   {
      UInt32 *src;
      UInt32 *dest;
      int row;


      src = GameApp.pictureBitmapBitsP[picture];
      dest = (UInt32 *) & GameApp.screenBufferP[y * GameApp.screenHeight + x];

      // Draw 15x15 16 bit bitmap quickly.
      for (row = 15; row > 0; row--)
      {
          *src++ = *dest++;
          *src++ = *dest++;
          *src++ = *dest++;
          *src++ = *dest++;
          *src++ = *dest++;
          *src++ = *dest++;
          *src++ = *dest++;
          *src++ = *dest++;
          *src++ = *dest++;
          *src++ = *dest++;
          *src++ = *dest++;
          *src++ = *dest++;
          *src++ = *dest++;
          *src++ = *dest++;
          *src++ = *dest++;
#if 0
          *((UInt16 *) src) = *((UInt16 *) dest);
          ((UInt16 *) src)++;
          ((UInt16 *) dest)++;
#endif
          (UInt8 *) dest += screenWidth;
      }

   }

}


/*! \brief	Draw an object at a specified location and mode

HostOpenScreenBuffer must be called first.

\see HostOpenScreenBuffer
\see HostDrawGameMenu
\see HostCloseScreenBuffer

*/

void HostDrawBackground()
{
	Err err;
	TwGfxRectType destRect;

	destRect.x = 0;
	destRect.y = 0;
	destRect.w = SCREEN_WIDTH;
	destRect.h = SCREEN_HEIGHT;

        // Copy the source window (contains the image to draw) to the draw window.
        err = TwGfxFillRect(GameApp.screenBufferH, &destRect, BackgroundColour);
	    ErrNonFatalDisplayIf(err != errNone, "TwGfxBitBlt problem");
}

void HostDrawBob(TwGfxPointType destPoint,TwGfxSurfaceType *curr_frame,TwGfxRectType source_rect)
{
	Err err;

        // Copy the source window (contains the image to draw) to the draw window.
        err = TwGfxBitBlt(GameApp.screenBufferH, &destPoint, curr_frame, &source_rect);
	    ErrNonFatalDisplayIf(err != errNone, "TwGfxBitBlt problem");

}
void HostDrawTransparentBob(TwGfxPointType destPoint,TwGfxSurfaceType *curr_frame,TwGfxRectType source_rect,TwGfxPackedRGBType Transparent_Colour)
{
	Err err;

        // Copy the source window (contains the image to draw) to the draw window.
        err = TwGfxTransparentBlt(GameApp.screenBufferH, &destPoint, curr_frame, &source_rect,Transparent_Colour);
	    ErrNonFatalDisplayIf(err != errNone, "TwGfxBitBlt problem");

}
void HostDrawScaledBob(TwGfxRectType destRect,TwGfxSurfaceType *curr_frame,TwGfxRectType source_rect, BOB_PTR bob)
{
	Err err;
	TwGfxRectType tempRect;
	
	tempRect.x = 0;
	tempRect.y = 0;
	tempRect.w = destRect.w;
	tempRect.h = destRect.h;
	
    TwGfxLockSurface(TempBufferH, (void**)&TempBufferP);

    // Copy the source window (contains the image to draw) to the draw window.
    err = TwGfxStretchBlt(TempBufferH, &tempRect, curr_frame, &source_rect);
    TwGfxUnlockSurface(TempBufferH, true);    // DOLATER true needed?

        
	if (bob->IsTransparent == 1)
        err = TwGfxTransparentBlt(GameApp.screenBufferH, &destRect, TempBufferH, &tempRect,bob->Transparent);
	else
        err = TwGfxBitBlt(GameApp.screenBufferH, &destRect, TempBufferH, &tempRect);
	   
	ErrNonFatalDisplayIf(err != errNone, "TwGfxBitBlt problem");

}
void HostFontDrawChars(char *TextIn, int textLength, int x, int y)
{
        FontDrawChars(TextIn, textLength, GameApp.screenBufferH, x, y);

}


void
HostDrawObject(int bitmapNumber,
    short int x,
    short int y)
{
	TwGfxRectType sourceBounds;
	TwGfxPointType destPoint;
	Err err;

    ErrNonFatalDisplayIf(GameApp.pictureBitmapP[bitmapNumber] == 0, "Unhandled object image");

    if (GameApp.customBlitting)
    {
        FastDrawPicture(bitmapNumber, x, y);
    }
    else
    {
        sourceBounds.x = 0;
        sourceBounds.y = 0;
        sourceBounds.w = HostGetBitmapWidth(bitmapNumber);
        sourceBounds.h = HostGetBitmapHeight(bitmapNumber);
        destPoint.x = x;
        destPoint.y = y;
        
        // Copy the source window (contains the image to draw) to the draw window.
        err = TwGfxBitBlt(GameApp.screenBufferH, &destPoint, GameApp.pictureBitmapP[bitmapNumber], &sourceBounds);
	    ErrNonFatalDisplayIf(err != errNone, "TwGfxBitBlt problem");
    }
}


/*! \brief Draw the game menu visible while the game is paused.

Usually called by the game when it knows it is paused.  This function 
is in the Host side so it can use the Palm OS fonts and drawing API.

\see HostOpenScreenBuffer
\see HostDrawGameMenu
\see HostCloseScreenBuffer

*/
#define FontAscent 4    /* the space between the top of a captial 
                           letter and the font box, sometimes occupied 
                           by accents. */
void
HostDrawGameMenu(int left,
    int top,
    int width,
    int height)
{
    int i;
    Int32 textLength;
    Int32 x;
    Int32 y;
    Int32 lineHeight;
    Int32 itemWidth;
    Int32 itemWidest = 0;
    TwGfxPointType lineEnds[4];
    
    
    // Pushing the state saves the current graphic modes before they
    // are set to what is needed.  They are popped after the drawing.
    // Pushing is a slow but complete method.  The alternative is to 
    // get and remember each setting.  There is a very limited number 
    // pushes allowed so do not nest.
    
    lineHeight = FontGetLineHeight();
//    lineHeight += lineHeight / 2;

    // Center the items vertically
    top += (int) (height - lineHeight * GameMenuCount - lineHeight / 3) / 2;

    for (i = 0; i < GameMenuCount; i++)
    {
        textLength = (Int16) StrLen(GameMenuText[i]);
        
        // Each item is drawn centered.
        itemWidth = 0;
        FontGetCharsInWidth(GameMenuText[i], &textLength, &itemWidth, NULL);
        if (itemWidth > itemWidest)
            itemWidest = itemWidth;
        x = left + width / 2 - itemWidth / 2;
        y = top + lineHeight * i;
        FontDrawChars(GameMenuText[i], textLength, 
            GameApp.screenBufferH, x, y);
    }
    
    // Show the selected item.  For now draw a line above and below
    // the item to indicate selection.
    x = left + width / 2 - itemWidest / 2;
    y = top + lineHeight * GameApp.gameMenuSelected + FontAscent;
    lineEnds[0].x = x;
    lineEnds[0].y = (Coord) (y - 1);
    lineEnds[1].x = (Coord) (x + itemWidest);
    lineEnds[1].y = (Coord) (y - 1);
    lineHeight = (lineHeight * 2) / 3;
    lineEnds[2].x = x;
    lineEnds[2].y = (Coord) (y + lineHeight);
    lineEnds[3].x = (Coord) (x + itemWidest);
    lineEnds[3].y = (Coord) (y + lineHeight);
    
    TwGfxDrawLineSegments(GameApp.screenBufferH, lineEnds, 4, ColorGameMenu);
}


/*! \brief	Stop draws to the screen buffer

\see HostOpenScreenBuffer
\see HostNewScreenBuffer
\see HostDrawObject
\see HostDrawBuffer
*/
void
HostCloseScreenBuffer(void)
{
/*	ErrNonFatalDisplayIf(GameApp.currentDrawWindowH == NULL, 
		"Close screen");
*/
    // Now unlock the screen buffer surface. 
    if (GameApp.customBlitting)
    {
        TwGfxUnlockSurface(GameApp.screenBufferH, true);    // DOLATER true needed?
        GameApp.screenBufferP = NULL;
    }
}


/* \brief  Return the time (ticks) to wait until the next game cycle.

\see AppHandleEvent
 */
static Int32
TimeUntillNextPeriod(void)
{
    Int32 timeRemaining = evtWaitForever;

	if (game_exists())
    {
        timeRemaining = (Int32) (GameApp.cycleNextTime - TwGetMicroSeconds());
        if (timeRemaining < 0)
        {
            timeRemaining = 0;
        }
    }

    return timeRemaining;
}


/*! \brief An event handler for keyDownEvents.
 *
 * \return    true if the event was handled and should not be passed
 *              to a higher level handler.
 */
SYSTEM_CALLBACK Boolean
AppHandleEvent(EventType * eventP)
{
    Boolean handled = false;

#ifndef OPTION_DELAY_NONE
    GTimeOut = TimeUntillNextPeriod();
#endif

    if (eventP->eType == nilEvent)
    {
        int startTime;
        int endTime;

        startTime = TwGetMicroSeconds();
            
        TwInputPoll(GameApp.inputQueueP, (Int32 *) &GameApp.joystickInput, sizeof(GameApp.joystickInput));
        game_advance(KeyCurrentState(), GameApp.joystickInput.joyX, GameApp.joystickInput.joyY, 
					GameApp.joystickInput.ActionA, GameApp.joystickInput.ActionB, GameApp.joystickInput.ActionC, 
					GameApp.joystickInput.ActionD, GameApp.joystickInput.TriggerA, GameApp.joystickInput.TriggerB);
            
        endTime = TwGetMicroSeconds();
        
        handled = true;
    }
    else if (eventP->eType == winExitEvent)
    {
        game_set_pause(game_pause);
        handled = false;
    }
    // Notice when drawing to the MainForm is safe again. Safe means
    // that our window is on top, and therefore not obscured.
    else if (eventP->eType == winEnterEvent)
    {
        game_set_pause(game_run);
        handled = false;
    }
    // If the launcher key is pressed, pause the game. 
    // If the game was already paused, then switch to the launcher.
    else if (eventP->eType == appStopEvent && game_get_pause() == game_run)
    {
        game_set_pause(game_pause);
        handled = true;
    }
    // Consume the hard keys, which other SysHandleEvent will see and
    // decide to switch to the app assigned to the key.
    else if (eventP->eType == keyDownEvent)
    {
        // Both the joystick and the action keys can generate a vchrPageDown
        if (eventP->data.keyDown.chr == vchrPageDown)
        {
            if (game_get_pause() == game_pause)
            {
                if (GameApp.gameMenuSelected < GameMenuCount - 1) {
                    GameApp.gameMenuSelected++;
                }
                handled = true;
            }
        }
        // Both the joystick and the action keys can generate a vchrPageUp
        else if (eventP->data.keyDown.chr == vchrPageUp)
        {
            if (game_get_pause() == game_pause)
            {
                if (GameApp.gameMenuSelected > 0) {
                    GameApp.gameMenuSelected--;
                }
                handled = true;
            }
        }
        // Act on the selected menu item.
        else if (eventP->data.keyDown.chr == vchrNavSelect ||
        	eventP->data.keyDown.chr == vchrFunction)
        {
            if (game_get_pause() == game_pause)
            {
                if (GameApp.gameMenuSelected == 0)
                {
                    game_set_pause(game_run);
                    
                    // Adjust the cycleNextTime which the pause changes.
                    GameApp.cycleNextTime = TwGetMicroSeconds() + GameApp.cycleDurationTime;
                }
                else if (GameApp.gameMenuSelected == 1)
                {
                    EventType newEvent;
                    // Enqueue an appStopEvent to exit this app. The
                    // OS switches to the last app, which typically is
                    // a launcher.
                	MemSet(&newEvent, sizeof(newEvent), 0);
                	newEvent.eType = appStopEvent;
                	EvtAddEventToQueue(&newEvent);
                }
            }

            handled = true;
        }
    }

    return handled;
}



/*! \brief The event loop for the aplication.  

The primary functions is to wait until the next game cycle and then
call the game to do it's thing.  But all other device input and activity 
is processed here too, like switching apps, turning off the device, 
alarms, and so on.
*/
static void
AppEventLoop(void)
{
    // Wait until the next game period.  This call either returns
    // immeadiately with an event, or blocks until the time requested
    // has passed, in which case it returns a nilEvent.
    // 
    // Palm OS does not have timers or threads or vertical syncs
    // available to signal a time to draw.  It only has machine ticks
    // which occur 1000 times per second while the device is awake.
    // 
    // To draw frames at a consistent interval we draw and then
    // perform work for the next frame that hopefully takes less time
    // than the desired frame rate permits.  If we finish early then
    // we wait the needed amount of ticks.  Otherwise we wait zero
    // ticks (but still allow the OS time for housekeeping).
    // 
    // An interesting topic is what to do when to much time is spent
    // preparing the next frame.  The two options are to do less work
    // or to slip a frame.
    // 
    // Doing less work can mean drawing less detail, less effects, or
    // lower quality drawing.  The idea is to not let the person
    // notice.  Applying lower quality to new visual elements can be
    // better than reducing something already seen.
    // 
    // Slipping a frame can be done occasionally without a noticeable
    // effect.  And when enough slippage happens it's the only way to
    // avoid a "rubberband" effect catching back up to the expected
    // time.  Skipping the drawing but not the world update is a good
    // way to recover some time but keep the world flowing smoothly.
    // It is a poor way to recover a lot of time because the player
    // feels out of control.
    // 
    // Not tracking the ticks, but simply waiting a constant time
    // makes the frame rate subject to the variability of the game
    // cycle.
    // 
    // It is important to qualify all game cycles for a typical game.
    // The game should have a mode to deterministically play through.
    // The number of ticks for each game cycle should be measured.
    // The average must be slightly less than the desired frame rate,
    // although a graph is much more informative.  The worst cases
    // should be recorded and replayed and examined to improve the
    // performance.  The worse game cycles are the most important,
    // because they cause the noticeable stuttering in otherwise silky
    // smooth games.
    //
    // The app stays in the loop above until an appStopEvent is
    // received by the user switching to another app.  Users of Palm
    // apps do not think of exiting an app, they simply go to another
    // app, and the current app gets an appStopEvent.

    TwAppRun(&GTimeOut);
}


/*! \brief Load the graphic resources needed for the game.

Load the graphics into memory.  Custom blitting info is setup if needed too.  This
version uses gzipped BMP bitmaps.

*/
static void
AppLoadGraphics(App * app)
{
    int i;
    Err error;

    
	// Keep the Object graphics locked because they are frequently used
    /* Bitmaps can be either Palm specific resources or custom resources.

       Palm specific resources are fine because there is tool support and 
       the blitting code is already written and hardware accelerated.  
       For an example using them, see the GoCart example.
       
       This code shows how to use a custom graphics format.  This example
       uses images in the BMP format because it supported by many tools.
       Since BMP images are not compressed when they have more than 256 
       colors, they are compressed using gzip on the desktop.  The
       images are listed in the RCP file and so included into the prc.
       
       Other formats could certainly be used, and might be perform better.
       So explore!
	*/
	
	/* This is a simple scheme that loads all graphics.  A smarter scheme 
	   would load the graphics only as they are needed.  This would reduce
	   memory usage by keeping less graphics around at any given point.  
	   This would also reduce load times by spreading the graphic loads
	   through out time.
	*/
    app->customBlitting = false;
    for (i = 0; i < ObjectBitmapCount; i++)
    {
        void * image;
        void * imageUncompressed;
        Int32 imageSize;
        MemHandle imageH;
        
        imageH = DmGetResource('BMPf', (UInt16) (ObjectBitmapFirst + i));
        image = MemHandleLock(imageH);
        imageSize = MemHandleSize(imageH);
        
        // Some of the images are compressed using gzip.  BMP images > 256 colors
        // are not compressed.  GZip is quite effective, quick and portable and makes a 
        // good choice.
        imageUncompressed = gunzip(image, imageSize);
        
        error = AppReadBMP(app->GFX, &app->pictureBitmapP[i], 
        	imageUncompressed == NULL ? image : imageUncompressed, imageSize);
        
        if (imageUncompressed != NULL)
        	MemPtrFree(imageUncompressed);
        
        MemHandleUnlock(imageH);
        DmReleaseResource(imageH);
        
        // If custom blitting can be used, then cache some of the info on
        // the pictures that will be used often. 
        if (app->customBlitting)
        {
            // Pixels in TwGfxSurfaces are not accessible until the surface is
            // locked.  Leave locked to avoid repeating the locking overhead,
            // but unlock later.
            error = TwGfxLockSurface(app->pictureBitmapP[i], &app->pictureBitmapBitsP[i]);
#if 0
            app->pictureIsTransparent[i] = BmpGetTransparentValue(
               app->pictureBitmapP[i], &app->pictureTransparencyIndex[i]);
#endif
        }
    }

}

void InitTempBuffer()
{
    TwGfxSurfaceInfoType surfInfo;


    // Allocate a surface to hold the image
    surfInfo.size = sizeof(surfInfo);
    surfInfo.width = 80;
    surfInfo.height =80;
    surfInfo.location = twGfxLocationAcceleratorMemoryNoBackingStore;
    surfInfo.pixelFormat = twGfxPixelFormatRGB565;
    xassert(TwGfxAllocSurface(GameApp.GFX, &TempBufferH, &surfInfo));

}


/*! \brief Obtain and configure resources needed to start the game.

Sets the screen to 16 bit color mode.

\see AppLoadGraphics

*/
static Err
AppStart(App * app)
{
    Err error;
    TwGfxInfoType screenInfo;
    UInt32 featureValue;
	UInt16 index;
	Int16 ang;
	UInt32 Rand1,Rand2,Rand3;
		
	Rand1 = TimGetTicks();

    TapWaveSupport = (Boolean) (FtrGet(twFtrCreator, twFtrAPIVersion, &featureValue) == errNone);

	Rand2 = TimGetTicks();

	SysSetOrientation(sysOrientationLandscape);
	PINSetInputAreaState(pinInputAreaHide);
	StatHide();
	TwAppStart(AppHandleEvent);
    
    //AK Music handle
    //MusicH = DmGetResource('MIDI', (UInt16) (1052));
    //MusicP = MemHandleLock(MusicH);

    app->isInputMasked = false;
    app->joystickInput.joyX = 0;
    app->joystickInput.joyY = 0;
    Rand3 = TimGetTicks();

	rand_seed(Rand1,Rand2,Rand3);
    screenInfo.size = sizeof(screenInfo);
    error = TwGfxOpenNoSignV2(&app->GFX, &screenInfo);
    if (error) return error;

    app->screenWidth = screenInfo.displayWidth;
    app->screenHeight = screenInfo.displayHeight;
    
    error = TwGfxGetDisplaySurface(app->GFX, &app->displaySurface);

    // One goal of the game should be to display something as soon as possible.  If starting
    // the game or loading the graphics takes a noticeable time, it's probably good to display
    // a splash screen first.  This makes the app feel responsive.

    FontInit(app->GFX);
    
	//AppLoadGraphics(app);
    app->customBlitting = false;

	Game_Init(); // AK - Outpost startup



    ColorGameMenu = TwGfxComponentsToPackedRGB(239, 211, 16);


    // Now set up game info specific to this session.

    app->screenBufferH = NULL;
    
    app->gameMenuSelected = 0;
    
    // Do a frame rate of 33 fps.
    app->cycleDurationTime = SysTicksPerSecond() / 33 * 1000;
    
    
    TwDeviceOpen(&GameApp.rumblerH, "vibrator0", "w");
    
    GameApp.SPFGauge = GaugeOpen(screenInfo.displayWidth);
    GameApp.FPSGauge = GaugeOpen(screenInfo.displayWidth);
    
    app->fps.count = 0;    

    
    // Allocate a screen buffer one tile larger in both directions. 
    // Scrolling can cause part of the tiles on opposite ends to be 
    // visible at once.  Clipping is done when copying from the 
    // screen buffer to the screen.
    HostNewScreenBuffer((Coord) (app->screenWidth),
        (Coord) (app->screenHeight));
    game_new((int) app->screenWidth, (int) app->screenHeight, 0);
    app->cycleNextTime = TimGetTicks() + app->cycleDurationTime;

	// AK start music playing
	// Get master volume level
	soundAmp = TwSndGetVolume();
	//error = TwMidiPlaySmf(MusicP,NULL,true);
	
	InitTempBuffer(); //AK temp scratch buffer

    return errNone;                 // no error
}


/*! \brief Cleanup the app by releasing and restoring all resources.

Frees the screen buffer.  Releases all graphics.  Restores the keys.
Closes all open forms.
*/
static void
AppStop(App * app)
{
    int i;

	// Saving isn't implemented, but this is where it would be done.
	// Save the game automatically and restore to where the player left
	// off when the app returns.  The 90% case when starting the app
	// is to resume, using the Main menu is maybe the 10% case, so optimize
	// accordingly.
    game_delete();
    
    TwDeviceClose(GameApp.rumblerH);
    
    // Restore the keys. 
    HostUnmaskKeys();

    // Delete the screenBuffer
    if (GameApp.screenBufferH)
    {
        TwGfxFreeSurface(app->screenBufferH);
        app->screenBufferH = NULL;
    }
    
    TwGfxFreeSurface(TempBufferH);


    // Unlock and release the locked bitmaps
    /*for (i = 0; i < ObjectBitmapCount; i++)
    {
        TwGfxFreeSurface(app->pictureBitmapP[i]);
        app->pictureBitmapP[i] = 0;
    }*/
    
    GaugeClose(GameApp.SPFGauge);
    GaugeClose(GameApp.FPSGauge);
    
	// shutdown game and release all resources
	Game_Shutdown();
	
    
    // Close the library. This frees up the surfaces we loaded too.
    TwGfxClose(app->GFX);
    app->GFX = NULL;
    app->displaySurface = NULL;

	TwAppStop();
}


/*! \brief The main entry point for the application.

All Palm applications start at PilotMain.
*/
    
UInt32
PilotMain(
    UInt16 cmd,
    MemPtr cmdPBP,
    UInt16 launchFlags)
{
    Err error = errNone;

    /* Palm apps can start for a number of reasons, including to find
       data, to cleanup after a device reset, and so on. Only do
       something for a normal application launch. */
    if (cmd == sysAppLaunchCmdNormalLaunch)
    {
        error = AppStart(&GameApp);

        if (error == errNone)
        {
            AppEventLoop();
            AppStop(&GameApp);
        }
    }

    return error;
}
